; $Id: dm_calc_projection.pro,v 1.93.2.4 2015/05/20 19:30:42 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_calc_projection
;
; PURPOSE:
;  calculate projection
;  apply masking, rebinning, and detector efficiency adjustment to data
;
; CATEGORY:
;  dcs_mslice
;
; HISTORY:
;  02/2010: added NXSPE support - Andrei Savici (saviciat@ornl.gov)   
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  May, 2015
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

@dm_filetools       ;for dm_startend()
@dm_macs_sequencer  ;for dm_optimizekidstep()

;replace ll with kidlim[0], ul with kidlim[1], cn with mean(kidlim) in the string 
function dm_calckidney,string,kidlim
    retstr = (n_elements(kidlim) eq 0)
    if retstr then tmp0 = strtrim(string,2) else tmp0 = 'value='+string
    tmp1 = stregex(tmp0,'ll|ul|cn',/fold_case)
    while tmp1 ne -1 do begin
       tmp2 = strlowcase(strmid(tmp0,tmp1,2))
       case tmp2 of 
            'll' : tmp0 = strmid(tmp0,0,tmp1)+(retstr?'l_limit':'('+dm_to_string(kidlim[0])+')')+strmid(tmp0,tmp1+2)
            'ul' : tmp0 = strmid(tmp0,0,tmp1)+(retstr?'u_limit':'('+dm_to_string(kidlim[1])+')')+strmid(tmp0,tmp1+2)
            'cn' : tmp0 = strmid(tmp0,0,tmp1)+(retstr?'center':'('+dm_to_string(mean(kidlim))+')')+strmid(tmp0,tmp1+2)
       endcase
       tmp1 = stregex(tmp0,'ll|ul|cn',/fold_case)
    endwhile
    if retstr then return,tmp0
    ok = execute(tmp0,1,1)
    return,value
end

function dm_rangestring,values,step=step,unit=unit,tolerance=tolerance
    if n_elements(values) eq 0 then return,''
    if n_elements(step) eq 0   then step=0.0 else step=abs(step) 
    if n_elements(unit) eq 0   then unit=''
    if n_elements(tolerance) eq 0 then tolerance = step/1e4
    tmp =  values[uniq(values,sort(values))]
    n_tmp = n_elements(tmp)
    if step eq 0 then return,dm_to_string(tmp[0])+unit+((n_tmp gt 1)?' to '+dm_to_string(tmp[n_tmp-1])+unit:'')
    outstr = dm_to_string(tmp[0])+unit
    vstart = tmp[0] & cnt = 0L
    for i=1L,n_tmp-1 do begin
        cnt = cnt+1
        if abs(vstart+cnt*step-tmp[i]) gt tolerance then begin
           if abs(tmp[i-1]-vstart) gt tolerance then outstr = outstr+' to '+dm_to_string(tmp[i-1])+unit
           outstr = outstr+', '+dm_to_string(tmp[i])+unit
           vstart = tmp[i] & cnt=0L
        endif   
    endfor
    if abs(tmp[i-1]-vstart) gt tolerance then outstr = outstr+' to '+dm_to_string(tmp[i-1])+unit
    return,outstr
end

;This function calculates the MACS monitor lambda/2 correction
;parameter:
;     Ei:       [ndat], incident energy
;     monitor:  [ndat], corresponding monitor counts
;     cfx:      [ndat], corresponding CFX status 0-no filter, correction will be applied
;keywords:
;     extra keywords for poly_fit
;return:
;     [ndat], new monitor counts
function dm_macs_lambda2correction,Ei,monitor,cfx,_ref_extra=_extra
    index  = where(cfx eq 0,count)
    if count eq 0 then return,monitor
    ;factor is from Jose's Intensitiesl_l2_Sim_nov.txt file
    factor = [[2,   2.378025665d],[2.5, 1.793602670d],[3,   1.490934555d],[3.5, 1.252089079d],[4,   1.127126095d],[4.5, 0.959301163d],[5,   0.792585170d],[5.5, 0.649712929d],$
              [6,   0.582762512d],[6.5, 0.521160485d],[7,   0.483133752d],[7.5, 0.444736852d],[8,   0.416914118d],[8.5, 0.390288542d],[9,   0.362685682d],[9.5, 0.347918053d],$
              [10,  0.339471838d],[10.5,0.327112776d],[11,  0.325793541d],[11.5,0.305038794d],[12,  0.292795409d],[12.5,0.284034166d],[13,  0.264029518d],[13.5,0.268863169d],$
              [14,  0.259363234d],[14.5,0.247959631d],[15,  0.246067755d],[15.5,0.237768686d],[16,  0.225825977d],[16.5,0.228803554d],[17,  0.229217028d],[17.5,0.235338426d],$
              [18,  0.236590056d],[18.5,0.233790439d],[19,  0.242718897d],[19.5,0.242768485d],[20,  0.242982347d]]
    parm   = poly_fit(factor[0,*],factor[1,*],7,_extra=_extra)
    newmon = monitor
    newmon[index] = newmon[index]/(1.0+0.5*float(poly(Ei[index],parm)))
    return,newmon
end

;keywords:
;     temperature: if present, no prompt for temperautre, useful in scripts
;     dwu2:        if present, no prompt for Debye-Waller factro correction u^2, useful in scripts
pro dcs_mslice::dm_calc_projection,temperature=temperature,dwu2=dwu2,debug=debug
    self.error  = 1b    ;initialize the error flag, it will be cleared at the end of the program
    elastic_tol = 0.005 ;tolerance value for elastic energy
    if keyword_set(debug) then begin
       print,'**Calculate projection...'
       current = systime(/sec)
    end

    case self.instrname of
      'dcs':begin     ;dcs
            if (~ ptr_valid(self.detPosPtr)) then self.detPosPtr=dm_load_detpos(/dcs,adjust2th=self.tthcorr)
            end
      'macs':begin    ;macs
            if (~ ptr_valid(self.detPosPtr)) then self.detPosPtr=dm_load_detpos(/macs)
            end
      'spe':begin     ;spe
            if (~ ptr_valid(self.detPosPtr)) then begin
               self->dm_load_phx,phxfile=phxfile
               if strlen(phxfile) ne 0 then self.phxfile=phxfile
            endif
            end
      'nxspe':begin   ;nxspe
            if (~ ptr_valid(self.detPosPtr)) then begin
               self->dm_load_phx,phxfile=phxfile
            endif
            end
      'inx':begin     ;inx
            if (~ ptr_valid(self.detPosPtr)) then begin
               self.detPosPtr = dm_load_detpos(/neat)
               ok = dialog_message(['Detector information is not available, using NEAT instrument info instead.','',$
                    'Or you can load an existing INX file and clear data to save the detector information.'],title='Please note:',dialog_parent=self.tlb,/center)
            endif
            end
      'wand':begin    ;wand
            if (~ ptr_valid(self.detPosPtr)) then self.detPosPtr=dm_load_detpos(/wand,adjust2th=self.tthcorr)
            end
      else:
    endcase

    if (~ ptr_valid(self.detPosPtr)) then begin
       info = ['Detector information is not available. The projection calculation is aborted.']
       if ((self.instrname eq 'nxspe') or (self.instrname eq 'spe')) and (~ptr_valid(self.dataStrPtr)) then $
       info = [info,'','You can load an existing '+self.instrname+' file and clear data to save the detector information.']
       ok   = dialog_message(info,/error,dialog_parent=self.tlb,/center)
       return
    endif

    extravaxis = (self.extravaxis_yn[0] and (self.instrname ne 'macs') and (self.instrname ne 'wand'))

    ;check if the common input parameters are all set for single crystal and diffuse scattering type
    if self.samp_typ ne 0 then begin
       if total(finite(self.latt_parm,/nan)) ne 0 then begin
          mesg = 'Missing lattice parameters.'
       endif
       if total([finite(self.orie_u,/nan),finite(self.orie_v,/nan),finite(self.orie_psi,/nan)]) ne 0 then begin
          if n_elements(mesg) eq 0 then $
             mesg = 'Missing crystal orientation parameters.'  $
          else $
             mesg = [mesg,'Missing crystal orientation parameters.']
       endif
       if (total([finite(self.view_u1,/nan),finite(self.view_u2,/nan)]) ne 0) or ((total(finite(self.view_u3,/nan)) ne 0) and extravaxis) then begin
          if n_elements(mesg) eq 0 then $
             mesg = 'Missing viewing axes parameters.'  $
          else $
             mesg = [mesg,'Missing viewing axes parameters.']
       endif
       if n_elements(mesg) gt 0 then begin
          ok = dialog_message(mesg,/error,dialog_parent=self.tlb,/center)
          return
       endif
       if (self.instrname eq 'macs') or (self.instrname eq 'wand') and ~self->viewonscattplane() then begin
          ok = dialog_message('The viewing axes are not on the scattering plane.',dialog_parent=self.tlb,/center) ;not fatal, allow it to continue
       endif
    endif

    if (strlen(self.userf_str) ne 0) then begin ;remove comment lines, just in case
       userf_str = dm_strip_comments(strsplit(self.userf_str,'&',/extract),/combine)
    endif else userf_str = ''

    if (~ ptr_valid(self.dataStrPtr)) then begin    ;no data loaded yet
       if self.instrname eq 'macs' then begin
          eief = 5.0
          if (finite(self.eief)) then $
             if self.eief gt 0 then eief=dm_to_number(dm_to_string(self.eief,resolution=3))
          fts = ['from','to','step']
          name = ['fixed '+(['Ei','Ef'])[self.instrgeom]+' (meV):','E '+fts+' (meV):','Kidney '+fts+':','Optimize kidney step:']
          default = [dm_to_string([eief,self.estartend]),self.kidney[0:2],(['0','1'])[strmatch(self.kidney[3],'no',/fold_case)]]    
          info = ['Leave kidney range blank for full allowable kidney range.']
          if ~lmgr(/vm) then info = [info,'Use UL and LL for the upper and lower kidney limit,','and CN for the mean kidney center position at each energy.','For example: ul-5, ll+5, cn-4.']
          info = [info,'Kidney offset = '+dm_to_string(self.kidney_offset,resolution=3)+string('b0'XB)]
          if self.samp_typ eq 1 then begin
             name = [name,'A3 '+fts+':']
             default = [default,dm_to_string(self.psi[0:2])]
             if self.datr_yn then begin
                name    = [name,'Dark Angle:','Dark Angle Width:'] 
                default = [default,dm_to_string(self.datr_psi),dm_to_string(self.datr_width)]
                info    = [info,'Dark angle position is relative to the crystal axis u.','Counterclockwise rotation is positive.']
             endif
          endif
          notok = 1b
          is_droplist = bytarr(n_elements(name)) & is_droplist[7] = 1
          while notok do begin
               anstr = dm_dialog_input(name,default=default,dialog_parent=self.tlb,/align_center,info=info,cancel=cancel,droplist_content=ptr_new(['Yes','No']),is_droplist=is_droplist)
               ans = dm_to_number(anstr,/float)
               if keyword_set(cancel) then return
               if total(finite(ans,/nan)) eq total(finite(ans[[4,5,7]],/nan)) then begin
                  if ans[0] gt 0 then notok = 0b
                  if (total(finite(ans[4:5],/nan)) ne 0) and (ans[6] eq 0) then notok = (~strmatch(anstr[4],anstr[5],/fold_case))
               endif
          endwhile
          kid_fnt = finite(ans[4:5])
          if (~lmgr(/vm)) and stregex(anstr[4],'ul|ll|cn',/boolean,/fold_case) then kid_fnt[0] = 0  ;need to replace ul, ll, and cn
          if (~lmgr(/vm)) and stregex(anstr[5],'ul|ll|cn',/boolean,/fold_case) then kid_fnt[1] = 0
          ans[[3,6]] = abs(ans[[3,6]])
          if ans[2] lt ans[1] then ans[1:2] = ans[[2,1]]
          if ans[3] eq 0 then ans[2] = ans[1] $
          else if ans[1] eq ans[2] then ans[3] = 0
          if strmatch(anstr[4],anstr[5],/fold_case) and (strlen(anstr[4]) ne 0) then anstr[6] = '0'
          if kid_fnt[0] and kid_fnt[1] then begin
             if ans[5] lt ans[4] then anstr[4:5] = anstr[[5,4]]
             if ans[6] eq 0 then anstr[5] = anstr[4] $
             else if ans[4] eq ans[5] then anstr[6] = '0'
          endif
          eief           = ans[0]
          self.eief      = ans[0] 
          self.estartend = ans[1:3]
          self.kidney    = anstr[4:7]
          nen            = 1>((self.estartend[2] eq 0)?1:round(1+(self.estartend[1]-self.estartend[0])/self.estartend[2]))
          if self.samp_typ eq 1 then begin
             ans[10] = abs(ans[10])
             if ans[9] lt ans[8] then ans[8:9] = ans[[9,8]]
             if ans[10] eq 0 then ans[9] = ans[8] $
             else if ans[8] eq ans[9] then ans[10] = 0
             self.psi[0:2] = ans[8:10]
             na3 = 1>((self.psi[2] eq 0)?1:round(1+(self.psi[1]-self.psi[0])/self.psi[2]))  
             if self.datr_yn then begin
                self.datr_psi   = ans[11]
                self.datr_width = abs(ans[12])
             endif
          endif else na3 = 1
          for i=0L,nen-1L do begin
              tmp_en = self.estartend[0]+self.estartend[2]*i
              if self.instrgeom eq 0 then tmp_ei = eief $ ;direct geometry
              else tmp_ei = eief+tmp_en                   ;inverse geometry
              kidran = dm_to_number(self.kidney[0:1]) 
              kidlim = dm_getkidneyrange(tmp_ei,offset=self.kidney_offset)
              if ~kid_fnt[0] then begin
                 if (~lmgr(/vm)) and (strlen(anstr[4]) ne 0) then kidran[0] = dm_calckidney(anstr[4],kidlim) else kidran[0] = kidlim[0]
              endif
              if ~kid_fnt[1] then begin
                 if (~lmgr(/vm)) and (strlen(anstr[5]) ne 0) then kidran[1] = dm_calckidney(anstr[5],kidlim) else kidran[1] = kidlim[1]
              endif
              kstep = dm_to_number(self.kidney[2])
              if strmatch(self.kidney[3],'yes',/fold_case) then kstep = dm_optimizekidtargetstep(kstep,kidran)
              nkid = 1>((kstep eq 0)?1:round(1+(kidran[1]-kidran[0])/kstep))
              for j=0L,nkid-1L do begin
                  tmp_kid = (kidran[0])>(kidran[0]+kstep*j)<(kidran[1])
                  if n_elements(a3) eq 0 then begin
                     a3     = self.psi[0]+self.psi[2]*findgen(na3)         
                     en     = replicate(tmp_en,na3)
                     Ei     = replicate(tmp_ei,na3)
                     kidney = replicate(tmp_kid,na3)
                  endif else begin
                     a3     = [a3,self.psi[0]+self.psi[2]*findgen(na3)]
                     en     = [en,replicate(tmp_en,na3)]
                     Ei     = [Ei,replicate(tmp_ei,na3)]
                     kidney = [kidney,replicate(tmp_kid,na3)]
                  endelse
              endfor
          endfor
          nen = n_elements(en)
          Ef  = Ei-en
          psi = 90.0-temporary(a3)
          ;check kidney
          index = dm_check_macskidney(kidney,Ei,count,complement=bindex,offset=self.kidney_offset)
          if (count eq 0) then begin
             ok = dialog_message('All kidney angles are out of valid range.',/error,dialog_parent=self.tlb,/center)
             return 
          endif else if count ne nen then begin
             bad_en  = en[bindex]
             bad_kid = kidney[bindex]
             Ei  = Ei[index]
             Ef  = Ef[index]
             en  = en[index]
             psi = psi[index]
             kidney = kidney[index]
             tmp = bad_en[uniq(bad_en,sort(bad_en))]
             if n_elements(mesg) eq 0 then mesg = 'The following kidney angles are out of valid range.' $
             else mesg = [mesg,'','The following kidney angles are out of valid range.']
             for i=0L,n_elements(tmp)-1L do begin
                 index = where(bad_en eq tmp[i])
                 tmpmin = min(bad_kid[index],max=tmpmax)
                 mesg = [mesg,'E='+dm_to_string(tmp[i])+' meV, kidney='+dm_rangestring(bad_kid[index],step=dm_to_number(self.kidney[2]),unit=string('b0'XB),tolerance=0.0001)]
             endfor
             mesg = [mesg,'These kidney angles have been removed from projection calculation.','']
             nen = count
          endif
          index = 0  & bindex = 0
       endif else begin
          ;ask for Ei or Ef according to instrument geometry
          eief = 5.0
          if (finite(self.eief)) then $
             if self.eief gt 0 then eief=self.eief
          if self.samp_typ eq 2 then begin             ;diffuse scattering single E
             case self.e_bin[self.samp_typ] of
               0:  en = 0.0                                      ;use elastic energy for all time channels
               1:  en = 0.0                                      ;use elastic energy for elastic peak
               2:  en = (self.e_range[0]+self.e_range[1])/2.0    ;use the mean energy for specified energy range
               else: en = 0.0
             endcase
             askeran = 0b
             if self.instrname eq 'wand' then $
                info = ['detector offset angle='+dm_to_string(-(*self.detPosPtr).two_theta[0],resolution=3)+string('b0'XB)] $
             else $
                info = ['E='+dm_to_string(en)+' meV']
          endif else if self.samp_typ eq 3 then begin  ;diffuse scattering multiple E
             askeran = 1b
             case self.e_bin[3] of
               0:  info = (['-Ei< E <Ei','-Ef< E <Ef'])[self.instrgeom]
               1:  begin
                   info = ['E~0 meV']
                   askeran = 0b
                   end
               2:  begin
                   info = [dm_to_string(self.e_range[0])+'< E <'+dm_to_string(self.e_range[1])]
                   if strlen(userf_str) ne 0 then self.estartend[0:1] = self.e_range[0:1]
                   end
               else:
             endcase
          endif else begin
             info = (['-Ei< E <Ei','-Ef< E <Ef'])[self.instrgeom]
             askeran = 1b
          endelse
          if (self.instrname eq 'dcs') or (self.instrname eq 'wand') then begin    ;for DCS convert energy(meV) to wavelength(angstrom)
             eief  = sqrt(81.8042/eief)
             title = "Incident wavelength:"
          endif else $
             title = (["Incident energy:","Final energy:"])[self.instrgeom]
          notok = 1b
          while notok do begin
             if askeran and strlen(userf_str) ne 0 then begin
                if (self.instrname eq 'dcs') or (self.instrname eq 'wand') then begin
                   eief = dm_dialog_input(['wavelength ('+string('c5'XB)+'):','E_start:','E_end:','n_E:'],/float,default=[finite(eief[0])?round(eief[0]*1000.0,/L64)/1000.0:eief[0],self.estartend],$
                          dialog_parent=self.tlb,info='E_start< E <E_end, n_E is the sampling number.',/lambda2e,title=title,/align_center)
                endif else begin
                   eief = dm_dialog_input([(['Ei','Ef'])[self.instrgeom]+'(meV):','E_start:','E_end:','n_E:'],/float,default=[finite(eief[0])?round(eief[0]*1000.0,/L64)/1000.0:eief[0],self.estartend],$
                          dialog_parent=self.tlb,info='E_start< E <E_end, n_E is the sampling number.',/e2lambda,title=title,/align_center)
                endelse
             endif else begin
                if (self.instrname eq 'dcs') or (self.instrname eq 'wand') then begin
                   eief = dm_dialog_input('wavelength ('+string('c5'XB)+'):',/float,default=finite(eief)?round(eief*1000.0,/L64)/1000.0:eief,dialog_parent=self.tlb,info=info,/lambda2e,title=title,/align_center)
                endif else begin
                   eief = dm_dialog_input((['Ei','Ef'])[self.instrgeom]+'(meV):',/float,title=title,default=finite(eief)?round(eief*1000.0,/L64)/1000.0:eief,dialog_parent=self.tlb,info=info,/e2lambda,/align_center)
                endelse
             endelse
             if (total(finite(eief)) eq n_elements(eief)) then $
                if eief[0] gt 0 then notok = 0b
          endwhile
          if (self.instrname eq 'dcs') or (self.instrname eq 'wand') then $    ;for DCS and WAND convert wavelength to energy
             eief[0] =  81.8042/(eief[0]^2)
          self.eief = eief[0]  ;save ei or ef information
          if askeran and strlen(userf_str) ne 0 then begin
             if eief[3] lt 1 then eief[3] = 81
             if self.instrgeom eq 0 then eief[2] = min([eief[2],eief[0]]) $
             else eief[1] = max([eief[1],-eief[0]])
             en = eief[1] + findgen(eief[3])/(eief[3]-1)*(eief[2]-eief[1]) +(2*self.instrgeom-1)*1e-6
             self.estartend = eief[1:3]
             eief = eief[0]
          endif else begin
             nnen = 81.           ;sampling number
             if self.samp_typ eq 2 then begin             ;diffuse scattering single E
                if (self.instrgeom eq 0) and (en gt eief) then begin
                   ok = dialog_message('Incident energy is less than the energy transfer: E>Ei. E=0 is used.',/error,dialog_parent=self.tlb,/center)
                   en = 0.0
                endif
                if (self.instrgeom eq 1) and (en lt -eief) then begin
                   ok = dialog_message('Neutron energy gain is more than the final energy: E<-Ef. E=0 is used.',/error,dialog_parent=self.tlb,/center)
                   en = 0.0
                endif
             endif else if self.samp_typ eq 3 then begin  ;diffuse scattering multiple E
                case self.e_bin[3] of
                  0:  en = (findgen(nnen)/((nnen-1)/2.)-1)*eief+(2*self.instrgeom-1)*1e-6         ;[-Ei,...,Ei] or [-Ef,...Ef]
                  1:  begin
                      nnen = 21.
                      ;use empirical DCS energy resolution as the energy range
                      dE = 0.02008*eief^(1.4735)
                      en = (findgen(nnen)/((nnen-1)/2.)-1)*dE
                      end
                  2:  begin
                      if self.instrgeom eq 0 then begin
                         if min(self.e_range) gt eief then begin
                            ok = dialog_message('Incident energy is smaller than the specified energy range. Elastic peak is used.',/error,dialog_parent=self.tlb,/center)
                            ;use empirical DCS energy resolution as the energy range
                            dE = 0.02008*eief^(1.4735)
                            en = (findgen(nnen)/((nnen-1)/2.)-1)*dE
                         endif else begin
                            eran = [self.e_range[0],min([self.e_range[1],eief])]
                            en   = eran[0] + findgen(nnen)/(nnen-1)*(eran[1]-eran[0])-1e-6
                         endelse
                      endif else begin
                         if max(self.e_range) lt -eief then begin
                            ok = dialog_message('Neutron energy gain in the specified energy range is more than the final energy. Elastic peak is used.',/error,dialog_parent=self.tlb,/center)
                            ;use empirical DCS energy resolution as the energy range
                            dE = 0.02008*eief^(1.4735)
                            en = (findgen(nnen)/((nnen-1)/2.)-1)*dE
                         endif else begin
                            eran = [max(self.e_range[0],-eief),self.e_range[1]]
                            en   = eran[0] + findgen(nnen)/(nnen-1)*(eran[1]-eran[0])+1e-6
                         endelse
                      endelse
                      end
                  else:
                endcase
             endif else begin
                en = (findgen(nnen)/((nnen-1)/2.)-1)*eief +(2*self.instrgeom-1)*1e-6         ;[-Ei,...,Ei] or [-Ef,...Ef]
             endelse
          endelse
          if self.samp_typ ge 2 then begin ;psi range for diffuse scattering type
             notok = 1b
             if total(finite(self.psi[0:2])) gt 0 then psi=self.psi[0:2] else psi = [0,50,2]
             name = ['Start:','End:','Step:']
             if self.datr_yn then begin
                psi    = [psi,self.datr_psi,self.datr_width]
                name   = [name,'Dark Angle:','Dark Angle Width:'] 
                dainfo = ['Dark angle position is relative to the crystal axis u.','Counterclockwise rotation is positive.']
             endif
             while notok do begin
                psi = dm_dialog_input(name,title='Psi range:',default=psi,/float,dialog_parent=self.tlb,info=dainfo)
                notok = (total(finite(psi[0:2],/nan)) ne 0)
             endwhile
             psi[2] = abs(psi[2])
             min_a  = min(psi[0:1],max=max_a)
             if psi[2] eq 0 then max_a = min_a $
             else if min_a eq max_a then psi[2] = 0
             self.psi[0:2] = [min_a,max_a,psi[2]]
             if self.datr_yn then begin
                self.datr_psi   = psi[3]
                self.datr_width = abs(psi[4])
             endif
             if psi[2] eq 0 then psi = min_a $
             else psi = min_a+findgen(fix((max_a-min_a)/psi[2])+1)*psi[2]
             n_file = n_elements(psi)
          endif
       endelse
       qty  = 0.
       err  = 0.
       ewid = 0.
       time = 0.
       data_temperature = 0.0 ;data temperature, for MACS and WAND
       data_hfield      = 0.0 ;magnetic field, for MACS
       data_specify     = 0.0 ;specified data, for MACS
    endif else begin
       data_temperature = (*self.dataStrPtr).temperature  ;data temperture, for MACS and WAND   
       data_hfield      = (*self.dataStrPtr).hfield       ;magnetic field, for MACS 
       data_specify     = (*self.dataStrPtr).specify      ;specified data, for MACS   
       if self.instrname eq 'macs' then begin
          qty = (*self.dataStrPtr).qty
          err = (*self.dataStrPtr).err
          Ei  = (*self.dataStrPtr).Ei
          Ef  = (*self.dataStrPtr).Ef
          psi = 90.0-((*self.dataStrPtr).rotation+self.orie_psi)  ;actually A3,self.orie_psi is the A3 offset angle, 90-a3 due to u definition and rotation definition  
          en  = Ei-Ef
          eief = self.eief
          kidney = (*self.dataStrPtr).kidney
          if self.macs_ptaionly then ptai = (*self.dataStrPtr).ptai
          if self.macs_lamb2 then weight = dm_macs_lambda2correction(Ei,(*self.dataStrPtr).weight[*,self.mon_typ[1]],(*self.dataStrPtr).cfx) $
          else weight = (*self.dataStrPtr).weight[*,self.mon_typ[1]]
          tmpind = where(weight eq 0,cnt)
          if cnt ne 0 then weight[tmpind] = 1.0
          monfac = (self.macs_mon[self.mon_sumfit[1],self.mon_typ[1]])/weight
          for i=0LL,n_elements(monfac)-1 do begin
              qty[*,*,i] = qty[*,*,i]*monfac[i]
              err[*,*,i] = err[*,*,i]*monfac[i]
          endfor
          if ptr_valid(self.bgdata) then begin
             bgqty = (*self.bgdata).qty
             bgerr = (*self.bgdata).err
             bgang = (*self.bgdata).rotation
             bgei  = (*self.bgdata).Ei
             bgef  = (*self.bgdata).Ef
             bgen  = (*self.bgdata).Ei-(*self.bgdata).Ef
             bgkid = (*self.bgdata).kidney
             nbgen = n_elements(bgen)
             if self.macs_lamb2 then weight = dm_macs_lambda2correction((*self.bgdata).Ei,(*self.bgdata).weight[*,self.mon_typ[1]],(*self.bgdata).cfx) $
             else weight = (*self.bgdata).weight[*,self.mon_typ[1]]
             tmpind = where(weight eq 0,cnt)
             if cnt ne 0 then weight[tmpind] = 1.0
             monfac = (self.macs_mon[self.mon_sumfit[1],self.mon_typ[1]])/weight
             for i=0LL,nbgen-1L do begin
                 bgqty[*,*,i] = bgqty[*,*,i]*monfac[i]
                 bgerr[*,*,i] = bgerr[*,*,i]*monfac[i]
             endfor
          endif
          if ptr_valid(self.bgdata2) then begin
             bgqty2 = (*self.bgdata2).qty
             bgerr2 = (*self.bgdata2).err
             bgang2 = (*self.bgdata2).rotation
             bgei2  = (*self.bgdata2).Ei
             bgef2  = (*self.bgdata2).Ef
             bgen2  = (*self.bgdata2).Ei-(*self.bgdata2).Ef
             bgkid2 = (*self.bgdata2).kidney
             nbgen2 = n_elements(bgen2)
             if self.macs_lamb2 then weight = dm_macs_lambda2correction((*self.bgdata2).Ei,(*self.bgdata2).weight[*,self.mon_typ[1]],(*self.bgdata2).cfx) $
             else weight = (*self.bgdata2).weight[*,self.mon_typ[1]]
             tmpind = where(weight eq 0,cnt)
             if cnt ne 0 then weight[tmpind] = 1.0
             monfac = (self.macs_mon[self.mon_sumfit[1],self.mon_typ[1]])/weight
             for i=0LL,nbgen2-1L do begin
                 bgqty2[*,*,i] = bgqty2[*,*,i]*monfac[i]
                 bgerr2[*,*,i] = bgerr2[*,*,i]*monfac[i]
             endfor
             ;apply the empty can factor
             if total(finite(self.empcanfactor)) eq 2 then begin
                bgqty  = bgqty*self.empcanfactor[0]
                bgerr  = bgerr*self.empcanfactor[0]
                bgqty2 = bgqty2*self.empcanfactor[1]
                bgerr2 = bgerr2*self.empcanfactor[1]
             endif
          endif
          monfac = 0
          ewid = 1
       endif else begin
          ewid = (*self.dataStrPtr).ewid 
          eief = (*self.dataStrPtr).eief & self.eief = eief
          qty  = (*self.dataStrPtr).qty
          err  = (*self.dataStrPtr).err
          while (~ finite(eief)) do begin
               eief = dm_dialog_input((['Ei','Ef'])[self.instrgeom]+'(meV):',/float,default=eief,dialog_parent=self.tlb,/e2lambda)
               (*self.dataStrPtr).eief = eief
               self.eief = eief
          endwhile
          en   = (*self.dataStrPtr).e 
          time = (*self.dataStrPtr).time
          if self.samp_typ ge 2 then begin                      ;diffuse scattering type
             psi = (*self.dataStrPtr).rotation                  ;angle 2-rotation
             self.orie_offset = self.orie_psi-self.psi[3]       ;save the offset information
             psi = psi+self.orie_offset                         ;adjust the offset
             n_file = n_elements(psi)
             self.psi[0:2] = dm_startend(psi,resolution=2)
          endif else if self.samp_typ eq 1 then begin
             if finite((*self.dataStrPtr).rotation) then $
                self.orie_offset = self.orie_psi-(*self.dataStrPtr).rotation
          endif
          if ptr_valid(self.bgdata) then begin
             bgqty = (*self.bgdata).qty
             bgerr = (*self.bgdata).err
             bgang = (*self.bgdata).rotation
             if self.samp_typ ge 2 then n_bgfile = n_elements(bgang)
          endif
          if ptr_valid(self.bgdata2) then begin
             bgqty2 = (*self.bgdata2).qty
             bgerr2 = (*self.bgdata2).err
             bgang2 = (*self.bgdata2).rotation
             if self.samp_typ ge 2 then n_bgfile2 = n_elements(bgang2)
             ;apply the empty can factor
             if total(finite(self.empcanfactor)) eq 2 then begin
                bgqty  = bgqty*self.empcanfactor[0]
                bgerr  = bgerr*self.empcanfactor[0]
                bgqty2 = bgqty2*self.empcanfactor[1]
                bgerr2 = bgerr2*self.empcanfactor[1]
             endif
          endif
       endelse
    endelse

    det_tt  = (*self.detPosPtr).two_theta 
    det_psi = (*self.detPosPtr).psi
    nen     = n_elements(en)
    ndet    = n_elements(det_tt)
   
    if self.intn_typ eq (n_elements(self.intn_typ_str)-2) then begin  ;J(Q,y), need to ask for mass
       Mjqy = !values.f_nan
       while (finite(Mjqy,/nan)) do begin
          Mjqy = dm_dialog_input('Mass:',/float,default=self.mjqy,dialog_parent=self.tlb,cancel=cancel,info='The mass is in atomic mass unit.',title=(['Enter mass:','Please confirm:'])[finite(self.mjqy)])
          if keyword_set(cancel) then return
       endwhile
       self.mjqy = Mjqy
    endif
   
    widget_control,/hourglass
    
    ;user function executed before empty can subtraction
    if (strlen(userf_str) ne 0) and (self.userf_before eq 2) then begin
       if self.userfwarning then begin
          if lmgr(/runtime) then usertmp = 'User Macro' else usertmp = 'User Macro/Function/Procedure'
          if n_elements(mesg) eq 0 then mesg = [usertmp+' is specified.',''] $
          else mesg = [usertmp+' is specified.','',mesg]
          self.userfwarning = 0b
       endif
       ok = execute(userf_str,1,1)
       if ~ok then begin
          errmsg = strmid(!error_state.msg,0,strlen(!error_state.msg)-1)+' in the user macro.'
          ok = dialog_message(errmsg,/error,dialog_parent=self.tlb,/center)
          return  ;error encountered
       endif
       if n_elements(err) ne n_elements(qty) then begin ;in case qty and err are not both defined in user functions
          if n_elements(err) eq 1 then begin
             err = replicate(err,size(qty,/dimension))
          endif else begin
             help,qty,err,output=tmp
             ok = dialog_message(['Incompatible array sizes of intensity and error bar, please check the user macro.','',tmp],/error,dialog_parent=self.tlb,/center)
             return
          endelse
       endif
    endif
    
    if ptr_valid(self.dataStrPtr) then begin
       if (self.samp_typ eq 0) and self.ftypecheckmask[0] and (self.instrname ne 'macs') then begin
          i = 0L
          while(total(finite(qty[i,*])) eq 0) do i=i+1    ;in case the first time channel is masked
          qtydet =  reform(qty[i,*])                      ;save the unadjust qty info for later mask determination
       endif
       ;subtract overall background rate
       if ptr_valid(self.bgratePtr) then begin
          bgrate = (*self.bgratePtr)
          nrate  = n_elements(bgrate)
          if nrate gt 1 then begin
             dbgrate = bgrate[*,1]
             bgrate  = bgrate[*,0]
          endif else $
             dbgrate = 1.0      ;arbitrary error bar
          case self.instrname of
            'dcs':begin   ;dcs
                  if nrate eq 1 then begin    ;overall rate
                     if ptr_valid(self.eff) then tmp_eff=(*self.eff) else tmp_eff=fltarr(ndet)+1.
                     tmp_value = bgrate/3600./(self.monrate)*temporary(tmp_eff)
                  endif else $
                     tmp_value = bgrate/3600./(self.monrate)
                  if ptr_valid(self.bgdata2) and total(finite(self.empcanfactor)) eq 2 then factor = self.empcanfactor else factor = [1.,1.]   
                  if self.samp_typ ge 2 then begin      ;diffuse scattering type
                     if self.samp_typ eq 2 then begin   ;single energy
                        if self.eint_yn then $
                           tmp_value = tmp_value*total(ewid) $
                        else $
                           tmp_value = tmp_value
                        for i=0L,n_file-1 do qty[*,i]=qty[*,i]-tmp_value
                        if ptr_valid(self.bgdata) then $
                           for i=0L,n_bgfile-1 do bgqty[*,i]=bgqty[*,i]-tmp_value*factor[0]
                        if ptr_valid(self.bgdata2) then $
                           for i=0L,n_bgfile2-1 do bgqty2[*,i]=bgqty2[*,i]-tmp_value*factor[1]
                     endif else begin                    ;multiple energy
                        for i=0L,n_file-1L do begin
                            for j=0L,nen-1L do $
                                qty[j,*,i] = qty[j,*,i]-tmp_value
                        endfor
                        if ptr_valid(self.bgdata) then begin
                           for i=0L,n_bgfile-1L do begin
                               for j=0L,nen-1L do $
                                   bgqty[j,*,i] = bgqty[j,*,i]-tmp_value*factor[0]
                           endfor
                        endif
                        if ptr_valid(self.bgdata2) then begin
                           for i=0L,n_bgfile2-1L do begin
                               for j=0L,nen-1L do $
                                   bgqty2[j,*,i] = bgqty2[j,*,i]-tmp_value*factor[1]
                           endfor
                        endif
                     endelse
                  endif else begin
                     for i=0L,nen-1L do $
                         qty[i,*] = qty[i,*]-tmp_value
                     if ptr_valid(self.bgdata) then begin
                        for i=0L,nen-1L do $
                            bgqty[i,*] = bgqty[i,*]-tmp_value*factor[0]
                     endif
                     if ptr_valid(self.bgdata2) then begin
                        for i=0L,nen-1L do $
                          bgqty2[i,*] = bgqty2[i,*]-tmp_value*factor[1]
                     endif
                  endelse
                  tmp_value = 1     ;free up the memory
                  end
            else:
          endcase
       endif
       ;subtract detector dependent background data
       if ptr_valid(self.bgdetsPtr) then begin
          tmp_value = (*self.bgdetsPtr)
          if self.samp_typ ge 2 then begin
             if self.eint_yn then $
                tmp_value = tmp_value*total(ewid)
             if self.samp_typ eq 2 then begin
                for i=0L,n_file-1L do $
                    qty[*,i] = qty[*,i]-tmp_value
             endif else begin
                tmp_value = (fltarr(nen)+1.0)#tmp_value
                for i=0L,n_file-1L do $
                    qty[*,*,i] = qty[*,*,i]-tmp_value
             endelse
          endif else begin
             qty = qty-(fltarr(nen)+1.0)#tmp_value
          endelse
          tmp_value = 1     ;free up the memory
       endif
       ;subtract time channel background data
       if ptr_valid(self.bgtchnPtr) then begin
          tmp_emin  = min((*self.bgtchnPtr).emean,max=tmp_emax)
          tmp_nen   = n_elements((*self.bgtchnPtr).emean)
          tchan_ind = where(en ge tmp_emin and en le tmp_emax,tchan_count)
          if tchan_count gt tmp_nen then tchan_ind = tchan_ind[0:(tmp_nen-1)]   ;errorproof
          if tchan_count ne 0 then begin
             if self.samp_typ eq 3 then begin
                for i=0L,n_file-1L do $
                    for j=0L,ndet-1L do $
                        qty[tchan_ind,j,i] = qty[tchan_ind,j,i]-(*self.bgtchnPtr).qty
             endif else begin
                for i=0L,ndet-1L do $
                    qty[tchan_ind,i] = qty[tchan_ind,i]-(*self.bgtchnPtr).qty
             endelse
          endif
       endif else tchan_count = 0
       ;subtract empty can file
       empty_count = 0L  ;limit the number of empty can mismatch error messages
       if ptr_valid(self.bgdata) then begin
          tmp_nz_ind = where(qty ne 0,tmp_nz_cnt)
          if finite(self.ssfac) then ssfac=self.ssfac $
          else ssfac = 1.0     ;self-shielding factor
          if self.instrname eq 'macs' then begin
             tmp_tol = abs(self.macs_tolerance)
             for i=0,n_elements(tmp_tol)-1 do if tmp_tol[i] eq 0 then tmp_tol[i] = ([0.01,0.1,0.05])[i]
             for i=0L,nen-1 do begin
                 mismatch = 0b
                 if self.empt_tol_optn eq 0 then begin
                    if self.samp_typ eq 0 then $
                       ;index = where((abs(bgen-en[i]) le self.macs_tolerance[0]) and (abs(bgkid-kidney[i]) le self.macs_tolerance[1]),count) $
                       index = where((abs(bgei-Ei[i]) le self.macs_tolerance[0]) and (abs(bgef-Ef[i]) le self.macs_tolerance[0]) and (abs(bgkid-kidney[i]) le self.macs_tolerance[1]),count) $
                    else $
                       ;index = where((abs(bgen-en[i]) le self.macs_tolerance[0]) and (abs(bgkid-kidney[i]) le self.macs_tolerance[1]) and (abs(bgang-(*self.dataStrPtr).rotation[i]) le self.macs_tolerance[2]),count)
                       index = where((abs(bgei-Ei[i]) le self.macs_tolerance[0]) and (abs(bgef-Ef[i]) le self.macs_tolerance[0]) and (abs(bgkid-kidney[i]) le self.macs_tolerance[1]) and (abs(bgang-(*self.dataStrPtr).rotation[i]) le self.macs_tolerance[2]),count)
                    if count eq 0 then begin
                       if self.samp_typ eq 0 then begin
                          ;tmp1 = min(abs(bgen-en[i])/tmp_tol[0]+abs(bgkid-kidney[i])/tmp_tol[1],imin)
                          ;index = where((abs(bgen-en[i]) eq abs(bgen[imin]-en[i])) and (abs(bgkid-kidney[i]) eq abs(bgkid[imin]-kidney[i])))
                          tmp1 = min(abs(bgei-Ei[i])/tmp_tol[0]+abs(bgef-Ef[i])/tmp_tol[0]+abs(bgkid-kidney[i])/tmp_tol[1],imin)
                          index = where((abs(bgei-Ei[i]) eq abs(bgei[imin]-Ei[i])) and (abs(bgef-Ef[i]) eq abs(bgef[imin]-Ef[i])) and (abs(bgkid-kidney[i]) eq abs(bgkid[imin]-kidney[i])))
                       endif else begin
                          ;tmp1 = min(abs(bgen-en[i])/tmp_tol[0]+abs(bgkid-kidney[i])/tmp_tol[1]+abs(bgang-(*self.dataStrPtr).rotation[i])/tmp_tol[2],imin)
                          ;index = where((abs(bgen-en[i]) eq abs(bgen[imin]-en[i])) and (abs(bgkid-kidney[i]) eq abs(bgkid[imin]-kidney[i]) and (abs(bgang-(*self.dataStrPtr).rotation[i]) eq abs(bgang[imin]-(*self.dataStrPtr).rotation[i]))))
                          tmp1 = min(abs(bgei-Ei[i])/tmp_tol[0]+abs(bgef-Ef[i])/tmp_tol[0]+abs(bgkid-kidney[i])/tmp_tol[1]+abs(bgang-(*self.dataStrPtr).rotation[i])/tmp_tol[2],imin)
                          index = where((abs(bgei-Ei[i]) eq abs(bgei[imin]-Ei[i])) and (abs(bgef-Ef[i]) eq abs(bgef[imin]-Ef[i])) and (abs(bgkid-kidney[i]) eq abs(bgkid[imin]-kidney[i]) and (abs(bgang-(*self.dataStrPtr).rotation[i]) eq abs(bgang[imin]-(*self.dataStrPtr).rotation[i]))))
                       endelse
                       mismatch = 1b
                    endif
                 endif else begin
                    if self.samp_typ eq 0 then begin
                       ;tmp1 = min(abs(bgen-en[i])/tmp_tol[0]+abs(bgkid-kidney[i])/tmp_tol[1],imin)
                       ;index = where((abs(bgen-bgen[imin]) le self.macs_tolerance[0]) and (abs(bgkid-bgkid[imin]) le self.macs_tolerance[1]))
                       tmp1 = min(abs(bgei-Ei[i])/tmp_tol[0]+abs(bgef-Ef[i])/tmp_tol[0]+abs(bgkid-kidney[i])/tmp_tol[1],imin)
                       index = where((abs(bgei-bgei[imin]) le self.macs_tolerance[0]) and (abs(bgef-bgef[imin]) le self.macs_tolerance[0]) and (abs(bgkid-bgkid[imin]) le self.macs_tolerance[1]))
                    endif else begin
                       ;tmp1 = min(abs(bgen-en[i])/tmp_tol[0]+abs(bgkid-kidney[i])/tmp_tol[1]+abs(bgang-(*self.dataStrPtr).rotation[i])/tmp_tol[2],imin)
                       ;index = where((abs(bgen-bgen[imin]) le self.macs_tolerance[0]) and (abs(bgkid-bgkid[imin]) le self.macs_tolerance[1]) and (abs(bgang-bgang[imin]) le self.macs_tolerance[2]))
                       tmp1 = min(abs(bgei-Ei[i])/tmp_tol[0]+abs(bgef-Ef[i])/tmp_tol[0]+abs(bgkid-kidney[i])/tmp_tol[1]+abs(bgang-(*self.dataStrPtr).rotation[i])/tmp_tol[2],imin)
                       index = where((abs(bgei-bgei[imin]) le self.macs_tolerance[0]) and (abs(bgef-bgef[imin]) le self.macs_tolerance[0]) and (abs(bgkid-bgkid[imin]) le self.macs_tolerance[1]) and (abs(bgang-bgang[imin]) le self.macs_tolerance[2]))
                    endelse
                 endelse
                 if n_elements(index) eq 1 then begin
                    qty[*,*,i] = qty[*,*,i]-bgqty[*,*,index[0]]*ssfac
                    err[*,*,i] = sqrt(err[*,*,i]^2+(bgerr[*,*,index[0]])^2*(ssfac*ssfac))
                 endif else begin
                    tmp_qty = total(bgqty[*,*,index],3)
                    tmp_err = (bgerr[*,*,index])^2
                    if self.bin_checkzero and self.dataminzero[1,0] then begin
                       tmp_ind = where(bgqty[*,*,index] eq 0,count)
                       if count ne 0 then begin
                          zero_error = min(tmp_err[tmp_ind])
                          tmp_err[tmp_ind] = 0.0
                       endif
                       tmp_ind = 0
                    endif
                    tmp_err = total(tmp_err,3)
                    if self.bin_checkzero and self.dataminzero[1,0] then begin
                       tmp_ind = where(tmp_err eq 0,count)
                       if count ne 0 then tmp_err[tmp_ind] = zero_error
                    endif
                    qty[*,*,i] = qty[*,*,i]-tmp_qty/n_elements(index)*ssfac
                    err[*,*,i] = sqrt(err[*,*,i]^2+tmp_err/n_elements(index)*(ssfac*ssfac))
                 endelse
                 if mismatch then begin ;mismatch warning message
                    empty_count = empty_count+1
                    if empty_count le 30 then begin
                       if self.samp_typ eq 0 then $
                          mismatchmesg = 'No match for '+'E='+dm_to_string(en[i])+', kid='+dm_to_string(kidney[i])+', the closest empty can data (E='+dm_to_string(bgen[index],sep=',')+$
                             ',kid='+dm_to_string(bgkid[index],sep=',')+') is used.' $
                       else $
                          mismatchmesg = 'No match for '+'E='+dm_to_string(en[i])+', kid='+dm_to_string(kidney[i])+', A3='+dm_to_string((*self.dataStrPtr).rotation[i])+', the closest empty can data (E='+$
                             dm_to_string(bgen[index],sep=',')+',kid='+dm_to_string(bgkid[index],sep=',')+',A3='+dm_to_string(bgang[index],sep=',')+') is used.'
                       if n_elements(mesg) eq 0 then mesg = mismatchmesg else mesg=[mesg,mismatchmesg]
                    endif
                 endif
                 if keyword_set(debug) then print,'E='+dm_to_string(en[i])+','+dm_to_string(bgen[index])+' kid='+dm_to_string(kidney[i])+','+dm_to_string(bgkid[index])+' A3='+dm_to_string((*self.dataStrPtr).rotation[i])+','+dm_to_string(bgang[index])
             endfor
          endif else begin
             if self.samp_typ ge 2 then begin    ;diffuse scattering type
                if n_bgfile eq 1 then begin
                   if self.samp_typ eq 2 then begin
                      for i=0L,n_file-1L do begin
                          qty[*,i] = qty[*,i]-bgqty[*,0]*ssfac
                          err[*,i] = sqrt(err[*,i]^2+(bgerr[*,0])^2*(ssfac*ssfac))
                      endfor
                   endif else begin
                      for i=0L,n_file-1L do begin
                          qty[*,*,i] = qty[*,*,i]-bgqty[*,*,0]*ssfac
                          err[*,*,i] = sqrt(err[*,*,i]^2+(bgerr[*,*,0])^2*(ssfac*ssfac))
                          if tchan_count ne 0 then begin   ;avoid double-subtraction
                             qty[tchan_ind,*,i] = qty[tchan_ind,*,i]+bgqty[tchan_ind,*,0]*ssfac
                             ;error bar is not changed
                          endif
                      endfor
                   endelse
                endif else begin
                   for i=0L,n_file-1L do begin
                       mismatch = 0b
                       tmp1 = min(abs(bgang-(*self.dataStrPtr).rotation[i]),imin) & tmp1 = bgang[imin]
                       if self.empt_tol_optn eq 0 then begin
                          index = where(abs(bgang-(*self.dataStrPtr).rotation[i]) le self.psi_tolerance,count) 
                          if count eq 0 then begin
                             index = where(bgang eq tmp1)
                             mismatch = 1b
                          endif
                       endif else index = where(abs(bgang-tmp1) le self.psi_tolerance)
                       if self.samp_typ eq 2 then begin
                          if n_elements(index) eq 1 then begin
                             qty[*,i] = qty[*,i]-bgqty[*,index[0]]*ssfac
                             err[*,i] = sqrt(err[*,i]^2+(bgerr[*,index[0]])^2*(ssfac*ssfac))
                          endif else begin
                             tmp_qty = total(bgqty[*,index],2)
                             tmp_err = (bgerr[*,index])^2
                             if self.bin_checkzero and self.dataminzero[1,0] then begin
                                tmp_ind = where(bgqty[*,index] eq 0,count)
                                if count ne 0 then begin
                                   zero_error = min(tmp_err[tmp_ind])
                                   tmp_err[tmp_ind] = 0.0
                                endif
                                tmp_ind = 0
                             endif
                             tmp_err = total(tmp_err,2)
                             if self.bin_checkzero and self.dataminzero[1,0] then begin
                                tmp_ind = where(tmp_err eq 0,count)
                                if count ne 0 then tmp_err[tmp_ind] = zero_error
                             endif
                             qty[*,i] = qty[*,i]-tmp_qty/n_elements(index)*ssfac
                             err[*,i] = sqrt(err[*,i]^2+tmp_err/n_elements(index)*(ssfac*ssfac))
                          endelse
                       endif else begin
                          if n_elements(index) eq 1 then begin
                             qty[*,*,i] = qty[*,*,i]-bgqty[*,*,index[0]]*ssfac
                             err[*,*,i] = sqrt(err[*,*,i]^2+(bgerr[*,*,index[0]])^2*(ssfac*ssfac))
                             if tchan_count ne 0 then begin   ;avoid double-subtraction
                                qty[tchan_ind,*,i] = qty[tchan_ind,*,i]+bgqty[tchan_ind,*,index[0]]*ssfac
                                ;error bar is not changed
                             endif
                          endif else begin
                             tmp_qty = total(bgqty[*,*,index],3)
                             tmp_err = (bgerr[*,*,index])^2
                             if self.bin_checkzero and self.dataminzero[1,0] then begin
                                tmp_ind = where(bgqty[*,*,index] eq 0,count)
                                if count ne 0 then begin
                                   zero_error = min(tmp_err[tmp_ind])
                                   tmp_err[tmp_ind] = 0.0
                                endif
                                tmp_ind = 0
                             endif
                             tmp_err = total(tmp_err,3)
                             if self.bin_checkzero and self.dataminzero[1,0] then begin
                                tmp_ind = where(tmp_err eq 0,count)
                                if count ne 0 then tmp_err[tmp_ind] = zero_error
                             endif
                             qty[*,*,i] = qty[*,*,i]-tmp_qty/n_elements(index)*ssfac
                             err[*,*,i] = sqrt(err[*,*,i]^2+tmp_err/n_elements(index)*(ssfac*ssfac))
                             if tchan_count ne 0 then begin   ;avoid double-subtraction
                                qty[tchan_ind,*,i] = qty[tchan_ind,*,i]+tmp_qty/n_elements(index)*ssfac 
                                ;error bar is not changed
                             endif
                          endelse
                       endelse
                       if mismatch then begin ;mismatch warning message
                          empty_count = empty_count+1
                          if empty_count le 30 then begin
                             mismatchmesg = 'No match for psi='+dm_to_string((*self.dataStrPtr).rotation[i])+', the closest empty can data (psi='+dm_to_string(bgang[index],sep=',')+') is used.'
                             if n_elements(mesg) eq 0 then mesg = mismatchmesg else mesg=[mesg,mismatchmesg]
                          endif
                       endif
                   endfor
                endelse
             endif else begin
                qty = temporary(qty)-(bgqty)*ssfac
                err = sqrt(temporary(err)^2+(bgerr)^2*(ssfac*ssfac))
                if tchan_count ne 0 then begin
                   qty[tchan_ind,*] = qty[tchan_ind,*]+(bgqty[tchan_ind,*])*ssfac
                   ;error bar is not changed
                endif
             endelse
          endelse
          bgqty = 0 & bgerr = 0    ;free up the memory
       endif
       ;subtract empty can file set 2
       if ptr_valid(self.bgdata2) then begin
          if self.instrname eq 'macs' then begin
             for i=0L,nen-1 do begin
                 mismatch = 0b
                 if self.empt_tol_optn eq 0 then begin
                    if self.samp_typ eq 0 then $
                       ;index = where((abs(bgen2-en[i]) le self.macs_tolerance[0]) and (abs(bgkid2-kidney[i]) le self.macs_tolerance[1]),count) $
                       index = where((abs(bgei2-Ei[i]) le self.macs_tolerance[0]) and (abs(bgef2-Ef[i]) le self.macs_tolerance[0]) and (abs(bgkid2-kidney[i]) le self.macs_tolerance[1]),count) $
                    else $
                       ;index = where((abs(bgen2-en[i]) le self.macs_tolerance[0]) and (abs(bgkid2-kidney[i]) le self.macs_tolerance[1]) and (abs(bgang2-(*self.dataStrPtr).rotation[i]) le self.macs_tolerance[2]),count)
                       index = where((abs(bgei2-Ei[i]) le self.macs_tolerance[0]) and (abs(bgef2-Ef[i]) le self.macs_tolerance[0]) and (abs(bgkid2-kidney[i]) le self.macs_tolerance[1]) and (abs(bgang2-(*self.dataStrPtr).rotation[i]) le self.macs_tolerance[2]),count)
                    if count eq 0 then begin
                       if self.samp_typ eq 0 then begin
                          ;tmp1 = min(abs(bgen2-en[i])/tmp_tol[0]+abs(bgkid2-kidney[i])/tmp_tol[1],imin)
                          ;index = where((abs(bgen2-en[i]) eq abs(bgen2[imin]-en[i])) and (abs(bgkid2-kidney[i]) eq abs(bgkid2[imin]-kidney[i])))
                          tmp1 = min(abs(bgei2-Ei[i])/tmp_tol[0]+abs(bgef2-Ef[i])/tmp_tol[0]+abs(bgkid2-kidney[i])/tmp_tol[1],imin)
                          index = where((abs(bgei2-Ei[i]) eq abs(bgei2[imin]-Ei[i])) and (abs(bgef2-Ef[i]) eq abs(bgef2[imin]-Ef[i])) and (abs(bgkid2-kidney[i]) eq abs(bgkid2[imin]-kidney[i])))
                       endif else begin
                          ;tmp1 = min(abs(bgen2-en[i])/tmp_tol[0]+abs(bgkid2-kidney[i])/tmp_tol[1]+abs(bgang2-(*self.dataStrPtr).rotation[i])/tmp_tol[2],imin)
                          ;index = where((abs(bgen2-en[i]) eq abs(bgen2[imin]-en[i])) and (abs(bgkid2-kidney[i]) eq abs(bgkid2[imin]-kidney[i]) and (abs(bgang2-(*self.dataStrPtr).rotation[i]) eq abs(bgang2[imin]-(*self.dataStrPtr).rotation[i]))))
                          tmp1 = min(abs(bgei2-Ei[i])/tmp_tol[0]+abs(bgef2-Ef[i])/tmp_tol[0]+abs(bgkid2-kidney[i])/tmp_tol[1]+abs(bgang2-(*self.dataStrPtr).rotation[i])/tmp_tol[2],imin)
                          index = where((abs(bgei2-Ei[i]) eq abs(bgei2[imin]-Ei[i])) and (abs(bgef2-Ef[i]) eq abs(bgef2[imin]-Ef[i])) and (abs(bgkid2-kidney[i]) eq abs(bgkid2[imin]-kidney[i]) and (abs(bgang2-(*self.dataStrPtr).rotation[i]) eq abs(bgang2[imin]-(*self.dataStrPtr).rotation[i]))))
                       endelse
                       mismatch = 1b
                    endif
                 endif else begin
                    if self.samp_typ eq 0 then begin
                       ;tmp1 = min(abs(bgen2-en[i])/tmp_tol[0]+abs(bgkid2-kidney[i])/tmp_tol[1],imin)
                       ;index = where((abs(bgen2-bgen2[imin]) le self.macs_tolerance[0]) and (abs(bgkid2-bgkid2[imin]) le self.macs_tolerance[1]))
                       tmp1 = min(abs(bgei2-Ei[i])/tmp_tol[0]+abs(bgef2-Ef[i])/tmp_tol[0]+abs(bgkid2-kidney[i])/tmp_tol[1],imin)
                       index = where((abs(bgei2-bgei2[imin]) le self.macs_tolerance[0]) and (abs(bgef2-bgef2[imin]) le self.macs_tolerance[0]) and (abs(bgkid2-bgkid2[imin]) le self.macs_tolerance[1]))
                    endif else begin
                       ;tmp1 = min(abs(bgen2-en[i])/tmp_tol[0]+abs(bgkid2-kidney[i])/tmp_tol[1]+abs(bgang2-(*self.dataStrPtr).rotation[i])/tmp_tol[2],imin)
                       ;index = where((abs(bgen2-bgen2[imin]) le self.macs_tolerance[0]) and (abs(bgkid2-bgkid2[imin]) le self.macs_tolerance[1]) and (abs(bgang2-bgang2[imin]) le self.macs_tolerance[2]))
                       tmp1 = min(abs(bgei2-Ei[i])/tmp_tol[0]+abs(bgef2-Ef[i])/tmp_tol[0]+abs(bgkid2-kidney[i])/tmp_tol[1]+abs(bgang2-(*self.dataStrPtr).rotation[i])/tmp_tol[2],imin)
                       index = where((abs(bgei2-bgei2[imin]) le self.macs_tolerance[0]) and (abs(bgef2-bgef2[imin]) le self.macs_tolerance[0]) and (abs(bgkid2-bgkid2[imin]) le self.macs_tolerance[1]) and (abs(bgang2-bgang2[imin]) le self.macs_tolerance[2]))
                    endelse
                 endelse
                 if n_elements(index) eq 1 then begin
                    qty[*,*,i] = qty[*,*,i]-bgqty2[*,*,index[0]]*ssfac
                    err[*,*,i] = sqrt(err[*,*,i]^2+(bgerr2[*,*,index[0]])^2*(ssfac*ssfac))
                 endif else begin
                    tmp_qty = total(bgqty2[*,*,index],3)
                    tmp_err = (bgerr2[*,*,index])^2
                    if self.bin_checkzero and self.dataminzero[1,0] then begin
                       tmp_ind = where(bgqty2[*,*,index] eq 0,count)
                       if count ne 0 then begin
                          zero_error = min(tmp_err[tmp_ind])
                          tmp_err[tmp_ind] = 0.0
                       endif
                       tmp_ind = 0
                    endif
                    tmp_err = total(tmp_err,3)
                    if self.bin_checkzero and self.dataminzero[1,0] then begin
                       tmp_ind = where(tmp_err eq 0,count)
                       if count ne 0 then tmp_err[tmp_ind] = zero_error
                    endif
                    qty[*,*,i] = qty[*,*,i]-tmp_qty/n_elements(index)*ssfac
                    err[*,*,i] = sqrt(err[*,*,i]^2+tmp_err/n_elements(index)*(ssfac*ssfac))
                 endelse
                 if mismatch then begin ;mismatch warning message
                    empty_count = empty_count+1
                    if empty_count le 30 then begin
                       if self.samp_typ eq 0 then $
                          mismatchmesg = 'No match for '+'E='+dm_to_string(en[i])+', kid='+dm_to_string(kidney[i])+', the closest empty can data (E='+dm_to_string(bgen2[index],sep=',')+$
                             ',kid='+dm_to_string(bgkid2[index],sep=',')+') is used.' $
                       else $
                          mismatchmesg = 'No match for '+'E='+dm_to_string(en[i])+', kid='+dm_to_string(kidney[i])+', A3='+dm_to_string((*self.dataStrPtr).rotation[i])+', the closest empty can data (E='+$
                             dm_to_string(bgen2[index],sep=',')+',kid='+dm_to_string(bgkid2[index],sep=',')+',A3='+dm_to_string(bgang2[index],sep=',')+') is used.'
                       if n_elements(mesg) eq 0 then mesg = mismatchmesg else mesg=[mesg,mismatchmesg]
                    endif
                 endif
             endfor
          endif else begin
             if self.samp_typ ge 2 then begin    ;diffuse scattering type
                if n_bgfile2 eq 1 then begin
                   if self.samp_typ eq 2 then begin
                      for i=0L,n_file-1L do begin
                          qty[*,i] = qty[*,i]-bgqty2[*,0]*ssfac
                          err[*,i] = sqrt(err[*,i]^2+(bgerr2[*,0])^2*(ssfac*ssfac))
                      endfor
                   endif else begin
                      for i=0L,n_file-1L do begin
                          qty[*,*,i] = qty[*,*,i]-bgqty2[*,*,0]*ssfac
                          err[*,*,i] = sqrt(err[*,*,i]^2+(bgerr2[*,*,0])^2*(ssfac*ssfac))
                          if tchan_count ne 0 then begin   ;avoid double-subtraction
                             qty[tchan_ind,*,i] = qty[tchan_ind,*,i]+bgqty2[tchan_ind,*,0]*ssfac
                             ;error bar is not changed
                          endif
                      endfor
                   endelse
                endif else begin
                   for i=0L,n_file-1L do begin
                       mismatch = 0b
                       tmp1 = min(abs(bgang2-(*self.dataStrPtr).rotation[i]),imin) & tmp1 = bgang2[imin]
                       if self.empt_tol_optn eq 0 then begin
                          index = where(abs(bgang2-(*self.dataStrPtr).rotation[i]) le self.psi_tolerance,count)
                          if count eq 0 then begin
                             index = where(bgang2 eq tmp1)
                             mismatch = 1b
                          endif
                       endif else index = where(abs(bgang2-tmp1) le self.psi_tolerance)
                       if self.samp_typ eq 2 then begin
                          if n_elements(index) eq 1 then begin
                             qty[*,i] = qty[*,i]-bgqty2[*,index[0]]*ssfac
                             err[*,i] = sqrt(err[*,i]^2+(bgerr2[*,index[0]])^2*(ssfac*ssfac))
                          endif else begin
                             tmp_qty = total(bgqty2[*,index],2)
                             tmp_err = (bgerr2[*,index])^2
                             if self.bin_checkzero and self.dataminzero[1,0] then begin
                                tmp_ind = where(bgqty2[*,index] eq 0,count)
                                if count ne 0 then begin
                                   zero_error = min(tmp_err[tmp_ind])
                                   tmp_err[tmp_ind] = 0.0
                                endif
                                tmp_ind = 0
                             endif
                             tmp_err = total(tmp_err,2)
                             if self.bin_checkzero and self.dataminzero[1,0] then begin
                                tmp_ind = where(tmp_err eq 0,count)
                                if count ne 0 then tmp_err[tmp_ind] = zero_error
                             endif
                             qty[*,i] = qty[*,i]-tmp_qty/n_elements(index)*ssfac
                             err[*,i] = sqrt(err[*,i]^2+tmp_err/n_elements(index)*(ssfac*ssfac))
                          endelse
                       endif else begin
                          if n_elements(index) eq 1 then begin
                             qty[*,*,i] = qty[*,*,i]-bgqty2[*,*,index[0]]*ssfac
                             err[*,*,i] = sqrt(err[*,*,i]^2+(bgerr2[*,*,index[0]])^2*(ssfac*ssfac))
                             if tchan_count ne 0 then begin   ;avoid double-subtraction
                                qty[tchan_ind,*,i] = qty[tchan_ind,*,i]+bgqty2[tchan_ind,*,index[0]]*ssfac
                                ;error bar is not changed
                             endif
                          endif else begin
                             tmp_qty = total(bgqty2[*,*,index],3)
                             tmp_err = (bgerr2[*,*,index])^2
                             if self.bin_checkzero and self.dataminzero[1,0] then begin
                                tmp_ind = where(bgqty2[*,*,index] eq 0,count)
                                if count ne 0 then begin
                                   zero_error = min(tmp_err[tmp_ind])
                                   tmp_err[tmp_ind] = 0.0
                                endif
                                tmp_ind = 0
                             endif
                             tmp_err = total(tmp_err,3)
                             if self.bin_checkzero and self.dataminzero[1,0] then begin
                                tmp_ind = where(tmp_err eq 0,count)
                                if count ne 0 then tmp_err[tmp_ind] = zero_error
                             endif
                             qty[*,*,i] = qty[*,*,i]-tmp_qty/n_elements(index)*ssfac
                             err[*,*,i] = sqrt(err[*,*,i]^2+tmp_err/n_elements(index)*(ssfac*ssfac))
                             if tchan_count ne 0 then begin   ;avoid double-subtraction
                                qty[tchan_ind,*,i] = qty[tchan_ind,*,i]+tmp_qty/n_elements(index)*ssfac 
                                ;error bar is not changed
                             endif
                          endelse
                       endelse
                       if mismatch then begin ;mismatch warning message
                          empty_count = empty_count+1
                          if empty_count le 30 then begin
                             mismatchmesg = 'No match for psi='+dm_to_string((*self.dataStrPtr).rotation[i])+', the closest empty can data (psi='+dm_to_string(bgang2[index],sep=',')+') is used.'
                             if n_elements(mesg) eq 0 then mesg = mismatchmesg else mesg=[mesg,mismatchmesg]
                          endif
                       endif
                   endfor
                endelse
             endif else begin
                qty = temporary(qty)-(bgqty2)*ssfac
                err = sqrt(temporary(err)^2+(bgerr2)^2*(ssfac*ssfac))
                if tchan_count ne 0 then begin
                   qty[tchan_ind,*] = qty[tchan_ind,*]+(bgqty2[tchan_ind,*])*ssfac
                   ;error bar is not changed
                endif
             endelse
          endelse
          bgqty2 = 0 & bgerr2 = 0    ;free up the memory
       endif
       if n_elements(mismatchmesg) ne 0 then begin
          if empty_count gt 30 then mesg = [mesg,'...',dm_to_string(empty_count-30)+' more mismatch'+(['es are',' is'])[empty_count eq 31]+' not listed.']
          mesg = [mesg,'Try increasing the empty can subtraction tolerance value.','']
       endif
       if keyword_set(tmp_nz_cnt) then begin  ;distinguish background subtracted 0 intensity from true 0 count intensity
          tmp_zr_ind = where(qty[tmp_nz_ind] eq 0,tmp_zr_cnt)
          if tmp_zr_cnt ne 0 then qty[tmp_nz_ind[tmp_zr_ind]] = 1e-30 ; a very small value
          tmp_nz_ind = 0 & tmp_zr_ind = 0
       endif
       
       ;absorption correction
       if ptr_valid(self.absp_info) then begin
          ninfo = n_elements(*(self.absp_info))
          info  = [(*(self.absp_info))[0],(*(self.absp_info))[3:(ninfo-1)]]
          if info[0] ge 0 and info[0] lt n_elements(self.absp_npts) then begin
             npts = (*self.absp_npts[info[0]])[*,(*(self.absp_info))[1]]  
             if self.instrname eq 'macs' then begin
                current1 = systime(/sec)
                for i=0LL,nen-1L do begin
                    if info[0] eq 3 then  info[n_elements(info)-1] = (*(self.absp_info))[ninfo-1]+psi[i]-(90.0-self.psi[3]-self.orie_psi) ;flat plate geometry
                    tmp = dm_calc_shielding(info,([Ei[i],Ef[i]])[self.instrgeom],en[i],kidney[i]-76.0+det_tt,interp=(*(self.absp_info))[2],instrgeom=self.instrgeom,is_dcs=(self.instrname eq 'dcs'),npoints=npts,group_leader=self.tlb,$
                              stopped=stopped,title='Absorption Correction:'+dm_to_string(i+1)+'/'+dm_to_string(nen),/noquestion)
                    if i eq 0 then begin
                       time = (systime(/sec)-current1)*nen/60.
                       if time gt 1 then begin
                          ok = dialog_message(['The absorption calculation will take about '+dm_to_string(time,resolution=1)+' mintues. Do you still want to continue?',$
                               'Try reducing the number of sampling points.'],/question,dialog_parent=self.tlb)
                          if ok eq 'No' then stopped = 1b
                       endif
                    endif
                    if (~stopped) then begin
                       qty[0,*,i] = qty[0,*,i]*tmp
                       qty[1,*,i] = qty[1,*,i]*tmp
                       err[0,*,i] = err[0,*,i]*tmp
                       err[1,*,i] = err[1,*,i]*tmp
                    endif else $
                       break
                endfor   
             endif else begin
                if (self.samp_typ lt 2) or (info[0] ne 3) then $
                   tmp = dm_calc_shielding(info,eief,en,det_tt,interp=(*(self.absp_info))[2],instrgeom=self.instrgeom,is_dcs=(self.instrname eq 'dcs'),npoints=npts,group_leader=self.tlb,stopped=stopped)          
                if self.samp_typ ge 2 then begin    ;diffuse scattering type
                   if self.samp_typ eq 2 then begin
                      for i=0L,n_file-1L do begin
                          if info[0] eq 3 then begin ;flat plate in diffuse scattering mode
                             info[n_elements(info)-1] = (*(self.absp_info))[ninfo-1]+psi[i]-self.orie_psi
                             tmp = dm_calc_shielding(info,eief,en,det_tt,interp=(*(self.absp_info))[2],instrgeom=self.instrgeom,is_dcs=(self.instrname eq 'dcs'),npoints=npts,group_leader=self.tlb,$
                                   stopped=stopped,title='Absorption Correction:'+dm_to_string(i+1)+'/'+dm_to_string(n_file),noquestion=(i gt 0),timefactor=n_file)          
                          endif
                          if (~stopped) then begin
                             qty[*,i] = qty[*,i]*tmp
                             err[*,i] = err[*,i]*tmp
                          endif else $
                             break
                      endfor
                   endif else begin
                      for i=0L,n_file-1L do begin
                          if info[0] eq 3 then begin ;flat plate in diffuse scattering mode
                             info[n_elements(info)-1] = (*(self.absp_info))[ninfo-1]+psi[i]-self.orie_psi
                             tmp = dm_calc_shielding(info,eief,en,det_tt,interp=(*(self.absp_info))[2],instrgeom=self.instrgeom,is_dcs=(self.instrname eq 'dcs'),npoints=npts,group_leader=self.tlb,$
                                   stopped=stopped,title='Absorption Correction:'+dm_to_string(i+1)+'/'+dm_to_string(n_file),noquestion=(i gt 0),timefactor=n_file)          
                          endif
                          if (~stopped) then begin
                             qty[*,*,i] = qty[*,*,i]*tmp
                             err[*,*,i] = err[*,*,i]*tmp
                          endif else $
                             break
                      endfor
                   endelse
                endif else begin
                   if (~stopped) then begin
                      qty = temporary(qty)*tmp
                      err = temporary(err)*tmp
                   endif 
                endelse
             endelse
             if stopped then begin  ;change the menu to absorption->None
                self->my_menu_toggle,'absp'+['None','Cylinder','Annulus','Sphere','Flat'],0
                (*(self.absp_info))[0] = -1
                tmpmesg = 'Absorption correction is disabled.'
                if (self.samp_typ ge 2) and (i gt 0) then $ ;some intensity and error bars have been modified
                   tmpmesg = tmpmesg+' The intensities of some angles have been modified.'    
                if n_elements(mesg) eq 0 then $
                   mesg = [tmpmesg,''] $
                else $
                   mesg = [mesg,tmpmesg,'']  
             endif
             tmp = 1    ;free up the memory
             widget_control,/hourglass
          endif
       endif
       ;efficiency adjustment
       if ptr_valid(self.eff) then begin
          if self.instrname eq 'macs' then begin
             for i=0LL,nen-1L do begin
                 qty[*,*,i] = qty[*,*,i]*(*self.eff)
                 err[*,*,i] = err[*,*,i]*(*self.eff)
             endfor
          endif else begin
             if self.samp_typ ge 2 then begin    ;diffuse scattering type
                if self.samp_typ eq 2 then begin
                   for i=0L,n_file-1L do begin
                       qty[*,i] = qty[*,i]/(*self.eff)
                       err[*,i] = err[*,i]/(*self.eff)
                   endfor
                endif else begin
                   for i=0L,n_file-1L do begin
                       for j=0L,nen-1L do begin
                           qty[j,*,i] = qty[j,*,i]/(*self.eff)
                           err[j,*,i] = err[j,*,i]/(*self.eff)
                       endfor
                   endfor
                endelse
             endif else begin
                for i=0L,nen-1L do begin
                    qty[i,*] = qty[i,*]/(*self.eff)
                    err[i,*] = err[i,*]/(*self.eff)
                endfor
             endelse
          endelse
       endif
       ;Debye-Waller adjustment
       if (self.intn_typ eq 4) or (self.intn_typ eq 5) then begin   
          ;ask for u^2
          oldu2 = 0. & u2_existed = 0b
          if n_tags((*self.dataStrPtr).info) ne 0 then begin
             u2names = tag_names((*self.dataStrPtr).info)
             index  = where(u2names eq 'DWU2',count)
             if count ne 0 then begin
                oldu2 = (*self.dataStrPtr).info.dwu2
                u2_existed = 1b
             endif
          endif
          if n_elements(dwu2) ne 0 then begin
             notvalid = 0b
             dwu2 = dwu2[0]       
          endif else $
             notvalid = 1b
          while notvalid do begin
             dwu2 = dm_dialog_input('u^2:',/float,default=oldu2,dialog_parent=self.tlb,title='Debye-Waller correction:',info='exp(u^2*Q^2) will be applied to correct for the Debye-Waller factor.')
             widget_control,/hourglass
             if finite(dwu2) then $
                if dwu2 ge 0 then begin
                   if u2_existed then (*self.dataStrPtr).info.dwu2=dwu2
                   notvalid = 0b ;valid u^2 given
                endif
          endwhile
       endif
       ;kfactor adjustment
       if (self.intn_typ eq 1) or (self.intn_typ eq 3) or (self.intn_typ eq 4) or (self.intn_typ eq 5) then begin   ;detailed balance factor
          kb = 8.617342e-2    ;Boltzmann constant in mev.k^-1
          if self.instrname ne 'macs' then begin
             ;ask for a sample temperature
             old = 300. & t_existed = 0b
             if n_tags((*self.dataStrPtr).info) ne 0 then begin
                tnames = tag_names((*self.dataStrPtr).info)
                index  = where(tnames eq 'TEMPERATURE',count)
                if count ne 0 then begin
                   old = (*self.dataStrPtr).info.temperature
                   t_existed = 1b
                endif
             endif
             if n_elements(temperature) ne 0 then begin
                notvalid = 0b
                temp = temperature[0]       
             endif else $
                notvalid = 1b
             while notvalid do begin
                temp = dm_dialog_input('sample temperature:',/float,default=old,dialog_parent=self.tlb,title='Detailed blance calculation')
                widget_control,/hourglass
                if finite(temp) then $
                   if temp gt 0 then begin
                      if t_existed then (*self.dataStrPtr).info.temperature=temp
                      notvalid = 0b ;valid temperature given
                   endif
             endwhile
             ;calculate detailed balance factor
             dbal = exp(-en/(2.0*kb*temp))
          endif else begin
             dbal = exp(-en/(2.0*kb*data_temperature[*,self.macstemp_typ]))
          endelse
       endif
       case self.intn_typ of    ;same case for ((self.instrname eq 'nxspe') and (self.lamcorrnxspe eq 0)) 
          0:  if self.instrname eq 'macs' then $      ;S(Q,omega)
              tmp = sqrt(2.07212/Ef) $
              else $
              tmp = (*self.kfactor)^4                 ;S(Q,omega)
          1:  if self.instrname eq 'macs' then $      ;Symmetrized S(Q,omega)
              tmp = sqrt(2.07212/Ef)*dbal $
              else $
              tmp = (*self.kfactor)^4*dbal            
          2:  tmp = (*self.kfactor)^3                 ;d2sigma/dOmegadEf
          3:  if self.instrname eq 'macs' then $      ;Chi(Q,omega)
              tmp = sqrt(2.07212/Ef)*(1-dbal^2) $
              else $
              tmp = (*self.kfactor)^4*(1-dbal^2)      
          4:  tmp = (*self.kfactor)^4*(1-dbal^2)*en   ;GDOS
          5:  tmp = (*self.kfactor)^3*(1-dbal^2)*en   ;VDOS   
          6:  if self.instrname eq 'macs' then $      ;J(Q,y)
              tmp = sqrt(2.07212/Ef) $
              else $
              tmp = (*self.kfactor)^4
         else:tmp = 1.0                               ;no kfactor adjustment
       endcase
       if ((self.instrname eq 'nxspe') and (self.lamcorrnxspe eq 1)) then begin ;very VERY unlikely case
          case self.intn_typ of
            0:  tmp = (*self.kfactor)^3               ;S(Q,omega)
            1:  tmp = (*self.kfactor)^3*dbal          ;Symmetrized S(Q,omega)
            2:  tmp = (*self.kfactor)^2               ;d2sigma/dOmegadEf
            3:  tmp = (*self.kfactor)^3*(1-dbal^2)    ;Chi(Q,omega)
            4:  tmp = (*self.kfactor)^3*(1-dbal^2)*en ;GDOS
            5:  tmp = (*self.kfactor)^2*(1-dbal^2)*en ;VDOS   
            6:  tmp = (*self.kfactor)^3               ;J(Q,y)
           else:tmp = 1.0                             ;no kfactor adjustment
          endcase
       endif
       if ((self.instrname eq 'nxspe') and (self.lamcorrnxspe eq 2)) then begin
          case self.intn_typ of
            0:  tmp = (*self.kfactor)                 ;S(Q,omega)
            1:  tmp = (*self.kfactor)*dbal            ;Symmetrized S(Q,omega)
            2:  tmp = 1.0                             ;d2sigma/dOmegadEf
            3:  tmp = (*self.kfactor)*(1-dbal^2)      ;Chi(Q,omega)
            4:  tmp = (*self.kfactor)*(1-dbal^2)*en   ;GDOS
            5:  tmp = (1-dbal^2)*en                   ;VDOS   
            6:  tmp = (*self.kfactor)                 ;J(Q,y)
           else:tmp = 1.0                             ;no kfactor adjustment
          endcase
       endif
       if ((self.instrname eq 'nxspe') and (self.lamcorrnxspe eq 3)) then begin
          case self.intn_typ of
            0:  tmp = 1.0                             ;S(Q,omega)
            1:  tmp = dbal                            ;Symmetrized S(Q,omega)
            2:  tmp = 1.0/(*self.kfactor)             ;d2sigma/dOmegadEf
            3:  tmp = 1-dbal^2                        ;Chi(Q,omega)
            4:  tmp = (1-dbal^2)*en                   ;GDOS
            5:  tmp = (1-dbal^2)*en/(*self.kfactor)   ;VDOS   
            6:  tmp = 1.0                             ;J(Q,y)
           else:tmp = 1.0                             ;no kfactor adjustment
          endcase
       endif      
       if self.eint_yn and (self.samp_typ ne 2) then $
          tmp = temporary(tmp)*ewid
       ;energy dependent detector efficiency adjustment for direct geometry
       if (self.edep_yn eq 1) and (self.instrgeom eq 0) then begin
          he3_temp = 293.12                                                                     ;assume the temperature is 20C = 293 K
          sigma_acoeff = 1.01325*5333/1.38065/he3_temp/sqrt(81.8042/25.3)*self.edep_det[1]*1e-2 ;5333 barn is the he-3 absorption cross section
          sigma_s = 1.01325*6/1.38065/he3_temp*self.edep_det[1]*1e-2                            ;6 barn is the he-3 total scattering cross section
          if self.instrname eq 'macs' then begin
             wl1 = sqrt(81.8042/Ef)
          endif else begin
             wl0 = sqrt(81.8042/(*self.dataStrPtr).eief)                                        ;incident wavelength
             wl1 = wl0*(*self.kfactor)                                                          ;scattered wavelength
          endelse
          sumtd = sigma_acoeff*wl1+sigma_s
          ;deteff_inv=1.0/(1.0-exp(-det_thick*sumtd))
          deteff_inv = sumtd/(sumtd-sigma_s)/(1.0-exp(-self.edep_det[0]*sumtd))                 ;from Jeremy's Equation 9
          sumtd = 0 & wl1 = 0
          tmp = temporary(tmp)*temporary(deteff_inv)
       endif
       if self.samp_typ ge 2 then begin
          if (self.samp_typ eq 2) or (n_elements(tmp) eq 1) then begin      
             qty = temporary(qty)*tmp       ;tmp is a scalor here, qty is a [ndet,nfile] arrray
             err = temporary(err)*tmp
          endif else begin                  ;tmp is a [nen] array, qty is a [nen,ndet,nfile] array
             for i=0L,ndet-1L do begin
                 for j=0L,n_file-1L do begin
                     qty[*,i,j] = qty[*,i,j]*tmp
                     err[*,i,j] = err[*,i,j]*tmp
                 endfor
             endfor
          endelse
       endif else begin
          if n_elements(tmp) ne 1 then begin
             if self.instrname eq 'macs' then begin
                for i=0LL,ndet-1L do begin      ;tmp is a [nen] array, qty is a [2,ndet,nen] array, only apply the factor to the spec intensity
                    qty[0,i,*] = qty[0,i,*]*tmp
                    err[0,i,*] = err[0,i,*]*tmp
                endfor   
             endif else begin
                for i=0L,ndet-1L do begin      ;tmp is a [nen] array, qty is a [nen,ndet] array
                    qty[*,i] = qty[*,i]*tmp
                    err[*,i] = err[*,i]*tmp
                endfor
             endelse
          endif else begin
             qty = temporary(qty)*tmp
             err = temporary(err)*tmp
          endelse
       endelse
       tmp = 0    ;free up the memory
       ;energy rebin
       if (self.e_bin[self.samp_typ] gt 1) and (self.samp_typ lt 2) then begin
          dm_grid_ebin,self.e_bin[self.samp_typ],qty,err,en,debug=debug,avgsum=self.bin_avgsum
          dm_grid_ebin,self.e_bin[self.samp_typ],ewid
          if (self.samp_typ eq 0) and (self.instrname eq 'dcs') then $
             dm_grid_ebin,self.e_bin[self.samp_typ],time
       endif
       ;calculate dark angle correction factor
       if (self.samp_typ ge 2) and self.dcor_yn and ptr_valid(self.dcor_info) then begin
          tmp_qty = (*self.dcor_info).factor
          tmp_ang = (*self.dcor_info).psi
          tmp_fac = fltarr(n_file)
          for i=0L,n_file-1L do begin
              tmp1 = abs(tmp_ang-(*self.dataStrPtr).rotation[i])  ;find the closest angle index
              tmp1 = min(tmp1,index)    ;index = (0L)>(value_locate(tmp_ang,(*self.dataStrPtr).rotation[i])) ;sort tmp_ang first if using value_locate
              tmp_fac[i] = tmp_qty[index[0]]
          endfor
          if self.samp_typ eq 2 then begin
             for i=0L,n_file-1L do begin
                 qty[*,i] = qty[*,i]*(tmp_fac[i])
                 err[*,i] = err[*,i]*(tmp_fac[i])
             endfor
          endif else begin
             for i=0L,n_file-1L do begin
                 qty[*,*,i] = qty[*,*,i]*(tmp_fac[i])
                 err[*,*,i] = err[*,*,i]*(tmp_fac[i])
             endfor
          endelse
       endif 
    
       if (self.instrname eq 'macs') and self.macs_ptaionly then begin
          tmp = indgen(20)+1
          for i=0L,nen-1 do begin
              ind = where(tmp ne ptai[i])
              qty[*,ind,i] = !values.f_nan
              err[*,ind,i] = !values.f_nan
          endfor
       endif
    endif
    
    ;mask a3 for macs
    if (self.instrname eq 'macs') and ptr_valid(self.a3mask) then begin
       tmp = bytarr(nen)
       tmpstrs = (*self.a3mask)
       tmp_a3  = 90.0-psi-([0,self.orie_psi])[ptr_valid(self.dataStrPtr)]  ;from psi back to instrument a3
       ;check for > and >= expression
       repeat begin
          tmp1 = stregex(tmpstrs,'>(=*)[ '+string(9b)+']*(-?[0-9.]+)',/subexpr,length=len)
          if len[0] gt 0 then begin
             tmp2 = dm_to_number(strmid(tmpstrs,tmp1[2],len[2]))
             if len[1] gt 0 then ind = where(tmp_a3 ge tmp2,count) $  ;'>='
             else ind = where(tmp_a3 gt tmp2,count)                   ;'>'
             if count gt 0 then tmp[ind] = 1b
             tmpstrs = strmid(tmpstrs,0,tmp1[0])+strmid(tmpstrs,tmp1[0]+len[0],strlen(tmpstrs)-tmp1[0]-len[0])
          endif
       endrep until len[0] le 0 
       ;check for > and >= expression
       repeat begin  
          tmp1 = stregex(tmpstrs,'<(=*)[ '+string(9b)+']*(-?[0-9.]+)',/subexpr,length=len)
          if len[0] gt 0 then begin
             tmp2 = dm_to_number(strmid(tmpstrs,tmp1[2],len[2]))
             if len[1] gt 0 then ind = where(tmp_a3 le tmp2,count) $  ;'<='
             else ind = where(tmp_a3 lt tmp2,count)                   ;'<'
             if count gt 0 then tmp[ind] = 1b
             tmpstrs = strmid(tmpstrs,0,tmp1[0])+strmid(tmpstrs,tmp1[0]+len[0],strlen(tmpstrs)-tmp1[0]-len[0])
          endif
       endrep until len[0] le 0 
       ;check for : expression
       repeat begin  
          tmp1 = stregex(tmpstrs,'(-?[0-9.]+)[ '+string(9b)+']*:[ '+string(9b)+']*(-?[0-9.]+)',/subexpr,length=len)
          if len[0] gt 0 then begin
             tmp2 = dm_to_number(strmid(tmpstrs,tmp1[1],len[1]))
             tmp3 = dm_to_number(strmid(tmpstrs,tmp1[2],len[2]))
             ind = where((tmp_a3 ge tmp2) and (tmp_a3 le tmp3),count) 
             if count gt 0 then tmp[ind] = 1b
             tmpstrs = strmid(tmpstrs,0,tmp1[0])+strmid(tmpstrs,tmp1[0]+len[0],strlen(tmpstrs)-tmp1[0]-len[0])
          endif
       endrep until len[0] le 0 
       ;check for single values
       tmp1 = strsplit(tmpstrs,'[a-zA-Z '+string(9b)+',:<>]',/regex,/extract,count=count)
       for j=0,count-1 do begin
           ind = where(abs(tmp_a3-dm_to_number(tmp1[j])) lt 0.0001,count)
           if count gt 0 then tmp[ind] = 1b
       endfor
       ind = where(tmp eq 0,count,complement=complement,ncomplement=ncomplement) & tmp=0 &tmp_a3=0
       if count ne 0 then begin
          if count ne nen then begin
             Ei = Ei[ind]
             Ef = Ef[ind]
             en = en[ind]
             psi = psi[ind]
             kidney = kidney[ind]
             if ptr_valid(self.dataStrPtr) then begin
                qty = qty[*,*,ind]
                err = err[*,*,ind]
                data_temperature = data_temperature[ind,*]
                if self.extravaxis_yn[1] then data_hfield  = data_hfield[ind]
                if self.extravaxis_yn[2] then data_specify = data_specify[ind]
             endif
             nen = count
          endif
       endif else begin
          ok = dialog_message('All A3 were masked. The mask has been cleared.',dialog_parent=self.tlb,/center)
          self->clear_mask,/a3
       endelse   
    endif
    
    ;mask detectors
    ;detector number mask
    if ptr_valid(self.mask) then begin
       tmp = bytarr(ndet)
       if (self.instrname eq 'macs') and (self.masktype eq 1) then tmp = bytarr(nen*ndet)
       tmp[*(self.mask)] = 1b 
       ind = where(tmp eq 0,count,complement=complement,ncomplement=ncomplement) & tmp=0
       if count ne 0 then begin
          if ptr_valid(self.dataStrPtr) then begin
             if self.instrname eq 'macs' then begin
                case self.masktype of 
                   0: begin
                      qty = qty[*,ind,*]
                      err = err[*,ind,*]
                      end
                   1: if ncomplement ne 0 then begin
                      qty = reform(qty,2,nen*ndet,/overwrite)
                      err = reform(err,2,nen*ndet,/overwrite)
                      qty[*,complement] = !values.f_nan
                      err[*,complement] = !values.f_nan
                      qty = reform(qty,2,ndet,nen,/overwrite)
                      err = reform(err,2,ndet,nen,/overwrite)
                      complement = 0
                      end
                   else:
                endcase
             endif else begin
                if self.samp_typ ge 2 then begin
                   if self.samp_typ eq 2 then begin
                      qty = qty[ind,*]
                      err = err[ind,*]
                   endif else begin
                      qty = qty[*,ind,*]
                      err = err[*,ind,*]
                   endelse
                endif else begin
                   qty = qty[*,ind]
                   err = err[*,ind]
                endelse
                if (self.samp_typ eq 0) and self.ftypecheckmask[0] then qtydet = qtydet[ind]
             endelse
          endif
          if (self.masktype eq 0) or (self.instrname ne 'macs') then begin
             det_tt  = det_tt[ind]
             det_psi = det_psi[ind]
             ndet    = count
          endif
          ind = 0
       endif else all_masked = 1b
    endif
    ;angular range mask
    if ptr_valid(self.mask_ar) then begin
       tmp = bytarr(ndet)
       if (self.instrname eq 'macs') then tmp = bytarr(nen*ndet)
       if ptr_valid(self.dataStrPtr) or (self.instrname ne 'macs') then begin
          if self.instrname eq 'macs' then begin  ;data available
             tmp_det_tt = det_tt#(fltarr(nen)+1)
             for i=0L,ndet-1 do tmp_det_tt[i,*] = kidney-76.0+det_tt[i]
          endif else $
             tmp_det_tt = det_tt
          tmpstrs = *(self.mask_ar)
          ;check for > and >= expression
          repeat begin
             tmp1 = stregex(tmpstrs,'>(=*)[ '+string(9b)+']*(-?[0-9.]+)',/subexpr,length=len)
             if len[0] gt 0 then begin
                tmp2 = dm_to_number(strmid(tmpstrs,tmp1[2],len[2]))
                if len[1] gt 0 then ind = where(tmp_det_tt ge tmp2,count) $  ;'>='
                else ind = where(tmp_det_tt gt tmp2,count)                   ;'>'
                if count gt 0 then tmp[ind] = 1b
                tmpstrs = strmid(tmpstrs,0,tmp1[0])+strmid(tmpstrs,tmp1[0]+len[0],strlen(tmpstrs)-tmp1[0]-len[0])
             endif
          endrep until len[0] le 0 
          ;check for > and >= expression
          repeat begin  
             tmp1 = stregex(tmpstrs,'<(=*)[ '+string(9b)+']*(-?[0-9.]+)',/subexpr,length=len)
             if len[0] gt 0 then begin
                tmp2 = dm_to_number(strmid(tmpstrs,tmp1[2],len[2]))
                if len[1] gt 0 then ind = where(tmp_det_tt le tmp2,count) $  ;'<='
                else ind = where(tmp_det_tt lt tmp2,count)                   ;'<'
                if count gt 0 then tmp[ind] = 1b
                tmpstrs = strmid(tmpstrs,0,tmp1[0])+strmid(tmpstrs,tmp1[0]+len[0],strlen(tmpstrs)-tmp1[0]-len[0])
             endif
          endrep until len[0] le 0 
          ;check for : expression
          repeat begin  
             tmp1 = stregex(tmpstrs,'(-?[0-9.]+)[ '+string(9b)+']*:[ '+string(9b)+']*(-?[0-9.]+)',/subexpr,length=len)
             if len[0] gt 0 then begin
                tmp2 = dm_to_number(strmid(tmpstrs,tmp1[1],len[1]))
                tmp3 = dm_to_number(strmid(tmpstrs,tmp1[2],len[2]))
                ind = where((tmp_det_tt ge tmp2) and (tmp_det_tt le tmp3),count) 
                if count gt 0 then tmp[ind] = 1b
                tmpstrs = strmid(tmpstrs,0,tmp1[0])+strmid(tmpstrs,tmp1[0]+len[0],strlen(tmpstrs)-tmp1[0]-len[0])
             endif
          endrep until len[0] le 0 
          ;check for single values
          tmp1 = strsplit(tmpstrs,'[a-zA-Z '+string(9b)+',:<>]',/regex,/extract,count=count)
          for j=0,count-1 do begin
              ind = where(abs(tmp_det_tt-dm_to_number(tmp1[j])) lt 0.0001,count)
              if count gt 0 then tmp[ind] = 1b
          endfor
       endif else begin ;'macs' without data 
          if n_elements(mesg) eq 0 then $ 
             mesg = ['The detector angular range mask has not been applied.',''] $
          else $
             mesg = [mesg,'The detector angular range mask has not been applied.','']
       endelse
       ind = where(tmp eq 0,count,complement=complement,ncomplement=ncomplement) & tmp=0 & tmp_det_tt = 0
       if count ne 0 then begin
          if ptr_valid(self.dataStrPtr) then begin
             if (self.instrname eq 'macs') then begin
                if ncomplement ne 0 then begin
                   qty = reform(qty,2,nen*ndet,/overwrite)
                   err = reform(err,2,nen*ndet,/overwrite)
                   qty[*,complement] = !values.f_nan
                   err[*,complement] = !values.f_nan
                   qty = reform(qty,2,ndet,nen,/overwrite)
                   err = reform(err,2,ndet,nen,/overwrite)
                   complement = 0
                endif
             endif else begin
                if self.samp_typ ge 2 then begin
                   if self.samp_typ eq 2 then begin
                      qty = qty[ind,*]
                      err = err[ind,*]
                   endif else begin
                      qty = qty[*,ind,*]
                      err = err[*,ind,*]
                   endelse
                endif else begin
                   qty = qty[*,ind]
                   err = err[*,ind]
                endelse
                if (self.samp_typ eq 0) and self.ftypecheckmask[0] then qtydet = qtydet[ind]
             endelse
          endif
          if (self.instrname ne 'macs') then begin
             det_tt  = det_tt[ind]
             det_psi = det_psi[ind]
             ndet    = count
          endif
          ind = 0
       endif else all_masked = 1b
    endif
    if keyword_set(all_masked) then begin
       ok = dialog_message('All detectors were masked. The detector mask has been cleared.',dialog_parent=self.tlb,/center)
       self->clear_mask
    endif  
    
    if self.samp_typ ge 1 then begin
       ;direct -> reciprocal basis vectors
       abc_r = dm_basis_r(self.latt_parm,error=error)
       if error then return  ;error encountered, no further calcuation
       ;orthogonalize and normalize the crystal orientation axes
       if norm(crossp(self.orie_u,self.orie_v)) eq 0 then begin    ;parallel
          ok = dialog_message('Please check cyrstal orientation.',/error,dialog_parent=self.tlb,/center)
          return          ;no further calculation
       endif
       uvw      = dblarr(3,3)
       uvw[*,0] = abc_r#self.orie_u
       uvw[*,1] = abc_r#self.orie_v
       uvw[*,0] = uvw[*,0]/norm(uvw[*,0],/double)
       uvw[*,1] = uvw[*,1]-uvw[*,0]#(transpose(uvw[*,1])#uvw[*,0])
       uvw[*,1] = uvw[*,1]/norm(uvw[*,1],/double)
       for i=0,2 do uvw[i,2]=uvw[(i+1) mod 3,0]*uvw[(i+2) mod 3,1]-uvw[(i+2) mod 3,0]*uvw[(i+1) mod 3,1]
                 ;cross product, not using crossp() for higher precision
       ;rewrite reciprocal lattice parameters in the (u,v,w) frame
       abc_r = transpose(uvw)#(temporary(abc_r))
       if self.samp_typ eq 1 then begin
          if self.instrname eq 'macs' then begin
             if self.macsintn_typ ne 0 then begin   ;'DIFF' or 'SPEC+DIFF' intensity, force Ef=Ei
                Ef = Ei
                en[*] = 0.0
             endif
          endif else begin
             psi = self.orie_psi
          endelse
          type = 1
       endif else type = 2 
    endif

    ptr_free,self.projStrPtr,self.powd_line
  
    ;user function executed before projection
    if (strlen(userf_str) ne 0) and (self.userf_before eq 1) then begin
       if self.userfwarning then begin
          if lmgr(/runtime) then usertmp = 'User Macro' else usertmp = 'User Macro/Function/Procedure'
          if n_elements(mesg) eq 0 then mesg = [usertmp+' is specified.',''] $
          else mesg = [usertmp+' is specified.','',mesg]
          self.userfwarning = 0b
       endif
       ok = execute(userf_str,1,1)
       if ~ok then begin
          errmsg = strmid(!error_state.msg,0,strlen(!error_state.msg)-1)+' in the user macro.'
          ok = dialog_message(errmsg,/error,dialog_parent=self.tlb,/center)
          return  ;error encountered
       endif
       if n_elements(err) ne n_elements(qty) then begin ;in case qty and err are not both defined in user functions
          if n_elements(err) eq 1 then begin
             err = replicate(err,size(qty,/dimension))
          endif else begin
             help,qty,err,output=tmp
             ok = dialog_message(['Incompatible array sizes of intensity and error bar, please check the user macro.','',tmp],/error,dialog_parent=self.tlb,/center)
             return
          endelse
       endif
    endif
    
    max_en = max(en,min=min_en)
    if (self.samp_typ eq 2) then begin
       if finite(self.e_range[0]) then min_en = (min_en)<(self.e_range[0])
       if finite(self.e_range[1]) then max_en = (max_en)>(self.e_range[1])
    endif

    case self.samp_typ of
     0:begin  ;powder
       ndat = n_elements(det_tt)
       ;handle the inherent masks in SPE file data
       if ptr_valid(self.dataStrPtr) and self.ftypecheckmask[0] and (self.instrname ne 'macs') then begin
          ind = where(finite(qtydet),count) & qtydet=0
          if count ne ndat then begin
             qty     = qty[*,ind] 
             err     = err[*,ind]
             det_tt  = det_tt[ind]
             det_psi = det_psi[ind]
             ndat    = count
          endif
       endif
       if self.instrname eq 'macs' then begin
          Q0 = dm_proj_spec_samp(Ei,Ef,det_tt,det_psi,type=0,kidney=kidney,debug=debug)               ;Q[ndat,ndet]
          Q1 = dm_proj_spec_samp(Ei,Ei,det_tt,det_psi,type=0,kidney=kidney,debug=debug)               ;Q[ndat,ndet]
          Q  = transpose([[[transpose(Q0)]],[[transpose(Q1)]]])                                       ;Q[2,ndat,ndet] 0-spec 1-diff
       endif else begin
          if ~self.negative2th then det_tt = abs(temporary(det_tt))
          Q = dm_proj_spec_samp(eief,en,det_tt,det_psi,type=0,debug=debug,instrgeom=self.instrgeom)   ;Q[nen,ndet]
       endelse
       if ptr_valid(self.powd_qvalue) and ((min_en le elastic_tol) and (max_en ge -elastic_tol)) then begin  ;powder line
          if self.instrname eq 'macs' then $
             maxQ = max(dm_proj_spec_samp(Ei,Ei,det_tt,det_psi,type=0,kidney=kidney),min=minQ) $
          else $
             maxQ = max(abs(sqrt(eief/2.0721246)*sin(det_tt/2*!dtor)*2.),min=minQ) 
          yval = [-2e27,1e-20,2e27]
          for i=0L,n_elements(*self.powd_qvalue)-1 do begin
             if ((*self.powd_qvalue)[i] lt minQ) then continue
             if ((*self.powd_qvalue)[i] gt maxQ) then break
             if n_elements(powd_x) ne 0 then begin
                powd_x = [powd_x,(*self.powd_qvalue)[i],(*self.powd_qvalue)[i],(*self.powd_qvalue)[i]]
                powd_y = [powd_y,yval]
             endif else begin
                powd_x = [(*self.powd_qvalue)[i],(*self.powd_qvalue)[i],(*self.powd_qvalue)[i]]
                powd_y = yval 
             endelse
             yval[[0,2]] = -yval[[0,2]]
          endfor
          if n_elements(powd_x) ne 0 then self.powd_line=ptr_new([[powd_x],[powd_y]],/no_copy)
       endif
       end
     else:begin  ;single crystal and diffuse scattering
       if self.fold_type eq 1 then begin  ;symmetry rotation folding
          if self.view_u1fold then begin
             index = where(finite(*(self.view_u1foldcenter)),count)
             if count eq 0 then begin
                if n_elements(mesg) eq 0 then $
                   mesg = ['n-fold symmetry value is invalid.',''] $
                else $
                   mesg = [mesg,'n-fold symmetry value is invalid.','']  
             endif else begin
                n_fold = (*(self.view_u1foldcenter))[index[0]]
                if (self.symrot_type eq 2) and (n_fold gt 1) then begin
                   n_psi = n_elements(psi)
                   tmpqty = qty & tmperr = err & i=1
                   while (max(psi)-min(psi) lt 360) do begin
                      psi = [psi,psi[0:n_psi-1]+360/n_fold*i]
                      if self.instrname eq 'macs' then begin
                         Ei = [Ei,Ei[0:(n_psi-1)]] & Ef = [Ef,Ef[0:(n_psi-1)]] & en = [en,en[0:(n_psi-1)]] & kidney = [kidney,kidney[0:(n_psi-1)]]
                         if self.extravaxis_yn[0] then data_temperature = [data_temperature,data_temperature[0:(n_psi-1),*]]
                         if self.extravaxis_yn[1] then data_hfield      = [data_hfield,data_hfield[0:(n_psi-1)]]
                         if self.extravaxis_yn[2] then data_specify     = [data_specify,data_specify[0:(n_psi-1)]]
                      endif else begin
                         if (self.instrname eq 'wand') and self.extravaxis_yn[0] then data_temperature = [data_temperature,data_temperature[0:(n_psi-1)]]
                      endelse
                      if size(tmpqty,/n_dim) eq 2 then begin
                         qty = [[qty],[tmpqty]] & err = [[err],[tmperr]]
                      endif else begin
                         qty = [[[qty]],[[tmpqty]]] & err = [[[err]],[[tmperr]]]
                      endelse
                      i = i+1
                   endwhile
                   n_fold = 1
                   tmpqty = 0 & tmperr = 0
                   if self.instrname eq 'macs' then nen = n_elements(Ei)
                   if self.samp_typ ge 2 then    n_file = n_elements(psi)
                endif
             endelse
          endif
       endif
       ;reconstruct the viewing axis and check orthogonality
       view_u1 = self.view_u1
       view_u2 = self.view_u2
       if extravaxis then view_u3 = self.view_u3
       U1    = abc_r#view_u1
       U2    = abc_r#view_u2
       itmp  = where(abs(U1) lt 1e-15,count) & if count gt 0 then U1[itmp] = 0d   ;to avoid the floating underflow condition
       itmp  = where(abs(U2) lt 1e-15,count) & if count gt 0 then U2[itmp] = 0d   ;to avoid the floating underflow condition
       norm1 = norm(U1,/double) & norm2 = norm(U2,/double)
       if self.view_ortharbi eq 0 then begin    ;check orthogonality
          if abs(transpose(U1)#U2/norm1/norm2) gt 1e-8 then begin      ;not orthogonal
             ok = dialog_message('Viewing axes are not orthogonal.',/error,dialog_parent=self.tlb,/center)
             return  ;no further calculation
          endif
       endif else begin
          ;make sure the viewing axes are not parallel to each other
          if norm(crossp(view_u1,view_u2)) eq 0 then begin
             ok = dialog_message('Parallel viewing axes',/error,dialog_parent=self.tlb,/center)
             return  ;no further calculation
          endif
       endelse
       if n_elements(view_u3) eq 3 then begin
          U3    = abc_r#view_u3
          itmp  = where(abs(U3) lt 1e-15,count) & if count gt 0 then U3[itmp] = 0d  ;to avoid the floating underflow condition
          norm3 = norm(U3,/double)
          if self.view_ortharbi eq 0 then begin    ;check orthogonality
             if max(abs([transpose(U2)#U3/norm2/norm3,transpose(U3)#U1/norm3/norm1])) gt 1e-8 then begin   ;not orthogonal
                ok = dialog_message('Viewing axes are not orthogonal.',/error,dialog_parent=self.tlb,/center)
                return   ;no further calculation
             endif
          endif else begin
             ;make sure no viewing axes are parallel to each other
             if (norm(crossp(view_u1,view_u3)) eq 0) or (norm(crossp(view_u2,view_u3)) eq 0) then begin
                ok = dialog_message('Parallel viewing axes',/error,dialog_parent=self.tlb,/center)
                return   ;no further calculation
             endif else if abs(transpose(U1)#crossp(U2,U3)) lt 1e-15 then begin
                ok = dialog_message('Three viewing axes are in the same plane.',/error,dialog_parent=self.tlb,/center)
                return   ;no further calculation
             endif
          endelse
       endif else $
          U3 = [0d,0d,0d]
       
       if self.instrname eq 'macs' then begin
          Q = dm_proj_spec_samp(Ei,Ef,det_tt,det_psi,psi=psi,type=type,debug=debug,is_spe=self.psi_spe,kidney=kidney)
                                                               ;Q[3,ndat,ndet] in sample frame
       endif else begin
          Q = dm_proj_spec_samp(eief,en,det_tt,det_psi,psi=psi,type=type,debug=debug,is_spe=self.psi_spe,diffetyp=(self.samp_typ eq 3),instrgeom=self.instrgeom)
                                                               ;Q[3,nen,ndet,nfile] in sample frame
       endelse
       QV = dm_proj_view(Q,U1,U2,U3,no_copy=((self.intn_typ lt 4) or (self.intn_typ eq (n_elements(self.intn_typ_str)-1))),n_fold=n_fold,sr_type=self.symrot_type,debug=debug) ;project onto viewing axis

       ;dark angle trajectory or mask dark angle
       if ((self.samp_typ ge 2) or ((self.samp_typ eq 1) and (self.instrname eq 'macs'))) and finite(self.datr_psi) and finite(self.datr_width) and $
          ((self.datr_yn and (~ptr_valid(self.dataStrPtr))) or (self.mask_darkangle and ptr_valid(self.dataStrPtr))) then begin  
          if ~ptr_valid(self.dataStrPtr) then begin ;dark angle trajectory
             if self.instrname eq 'macs' then begin  ;special case, det range change with Ei
                tmp_nen = 1>((self.estartend[2] eq 0)?1:round(1+(self.estartend[1]-self.estartend[0])/self.estartend[2]))
                tmp_na3 = 1>((self.psi[2] eq 0)?1:round(1+(self.psi[1]-self.psi[0])/self.psi[2]))      
                tmp_a3  = self.psi[0]+findgen(tmp_na3)*self.psi[2]
                da_tt0  = -(90-tmp_a3+self.datr_psi)
                tmp_ndt = n_elements(det_tt)
                tmp_da3 = reform(tmp_a3#(fltarr(6)+1),tmp_na3*6)
                tmp_dtt = reform(da_tt0#(fltarr(6)+1)+(fltarr(tmp_na3)+1)#(((findgen(6)-2.5)/5)*self.datr_width),tmp_na3*6)    ;dark angle range [-0.5,0.5]*self.datr_width
                while (total(da_tt0 gt 180)  ne 0)  do da_tt0  = da_tt0-360*(da_tt0 gt 180)
                while (total(da_tt0 lt -180) ne 0)  do da_tt0  = da_tt0+360*(da_tt0 lt -180)
                while (total(tmp_dtt gt 180)  ne 0) do tmp_dtt = tmp_dtt-360*(tmp_dtt gt 180)
                while (total(tmp_dtt lt -180) ne 0) do tmp_dtt = tmp_dtt+360*(tmp_dtt lt -180)
                ind2 = where(abs((abs(da_tt0)-180)) le self.datr_width/2.0,count2,ncomplement=count1)
                da_en = 0.0 & da_ei = 0.0 & da_tt = 0.0 & tmp_psi = 0.0
                for j=0L,tmp_nen-1L do begin
                    tmp_en = self.estartend[0]+self.estartend[2]*j
                    if self.instrgeom eq 0 then tmp_ei = eief $  ;direct geometry
                    else tmp_ei = eief+tmp_en                    ;inverse geometry
                    ;figure out the detector angle range
                    kidran = dm_to_number(self.kidney[0:1]) 
                    kidlim = dm_getkidneyrange(tmp_ei,offset=self.kidney_offset)
                    if ~kid_fnt[0] then begin
                       if (~lmgr(/vm)) and (strlen(anstr[4]) ne 0) then kidran[0] = dm_calckidney(self.kidney[0],kidlim) else kidran[0] = kidlim[0]
                    endif
                    if ~kid_fnt[1] then begin
                       if (~lmgr(/vm)) and (strlen(anstr[5]) ne 0) then kidran[1] = dm_calckidney(self.kidney[1],kidlim) else kidran[1] = kidlim[1]
                    endif
                    kstep = dm_to_number(self.kidney[2])
                    if strmatch(self.kidney[3],'yes',/fold_case) then kstep = dm_optimizekidstep(kstep,kidran)
                    nkid = 1>((kstep eq 0)?1:round(1+(kidran[1]-kidran[0])/kstep))
                    for k=0L,nkid-1L do begin
                        tmp_kid = (kidran[0])>(kidran[0]+kstep*k)<(kidran[1])
                        tmp = dm_check_macskidney(tmp_kid,tmp_ei,count,offset=self.kidney_offset)
                        if count eq 0 then continue
                        if count2 ne 0 then begin
                           da_tt1 = reform((det_tt-76+tmp_kid)#(fltarr(count2)+1.0),tmp_ndt*count2)
                           da_en   = [da_en,replicate(tmp_en,count2*tmp_ndt)]
                           da_ei   = [da_ei,replicate(tmp_ei,count2*tmp_ndt)]
                           da_tt   = [da_tt,da_tt1]
                           tmp_psi = [tmp_psi,reform((fltarr(tmp_ndt)+1.0)#(90-tmp_a3[ind2]),tmp_ndt*count2)] 
                        endif   
                        if count1 ne 0 then begin
                           tmp_ran = tmp_kid-76+det_tt[[0,tmp_ndt-1]]+[-0.5,0.5]
                           tmp_ind = where((tmp_dtt ge tmp_ran[0]) and (tmp_dtt le tmp_ran[1]),count)
                           if count eq 0 then continue else begin
                              da_en   = [da_en,replicate(tmp_en,count)]
                              da_ei   = [da_ei,replicate(tmp_ei,count)]
                              da_tt   = [da_tt,tmp_dtt[tmp_ind]]
                              tmp_psi = [tmp_psi,90-tmp_da3[tmp_ind]] 
                           endelse
                        endif    
                    endfor
                endfor
                n_da = n_elements(da_tt) & tmp_da3 = 0 & tmp_dtt = 0
                if n_da gt 1 then begin
                   da_ei = da_ei[1:n_da-1] & da_en = da_en[1:n_da-1] & da_tt = da_tt[1:n_da-1] & tmp_psi = tmp_psi[1:n_da-1]
                   if self.macsintn_typ ne 0 then begin
                      daQ = dm_proj_spec_samp(da_ei,da_ei,76,0,psi=tmp_psi,type=type,is_spe=self.psi_spe,kidney=da_tt)
                      da_en = fltarr(n_da)
                   endif else begin
                      daQ = dm_proj_spec_samp(da_ei,da_ei-da_en,76,0,psi=tmp_psi,type=type,is_spe=self.psi_spe,kidney=da_tt)
                   endelse
                endif
                da_ei = 0
             endif else begin
                ;figure out the detector angle range
                tmp_det_tt = det_tt
                tmp_detpsi = det_psi
                if self.psi_spe then begin
                   pos_ind = where((abs(tmp_detpsi) gt 90) and (abs(tmp_detpsi) lt 270),n_pos,complement=neg_ind,ncomplement=n_neg)       
                   if n_pos gt 0 then pos_max = max(tmp_det_tt[pos_ind],min=pos_min)
                   if n_neg gt 0 then neg_max = max(-tmp_det_tt[neg_ind],min=neg_min)
                endif else begin
                   pos_ind = where(tmp_det_tt gt 0,n_pos,complement=neg_ind,ncomplement=n_neg)
                   if n_pos gt 0 then pos_max = max(tmp_det_tt[pos_ind],min=pos_min)
                   if n_neg gt 0 then neg_max = max(tmp_det_tt[neg_ind],min=neg_min)
                endelse
                pos_ind = 0 & neg_ind = 0 & tmp_det_tt = 0 & tmp_detpsi = 0
                for i=0L,n_elements(psi)-1 do begin
                    da_tt = -(psi[i]+self.datr_psi)
                    while da_tt gt 180 do da_tt = da_tt-360
                    while da_tt lt -180 do da_tt = da_tt+360
                    if abs((abs(da_tt)-180)) le self.datr_width/2.0 then begin     ;blocking the incident beam
                       da_tt  = det_tt 
                       da_psi = det_psi
                    endif else begin
                       da_tt  = da_tt+(findgen(11)-5.)/10.*self.datr_width         ;dark angle range [-0.5,0.5]*self.datr_width
                       da_psi = fltarr(11)
                       tmp_ind = where(da_tt gt 180,tmp_count)
                       while tmp_count gt 0 do begin
                          da_tt[tmp_ind] = da_tt[tmp_ind]-360
                          tmp_ind = where(da_tt gt 180,tmp_count)
                       endwhile
                       tmp_ind = where(da_tt lt -180,tmp_count)
                       while tmp_count gt 0 do begin
                          da_tt[tmp_ind] = da_tt[tmp_ind]+360
                          tmp_ind = where(da_tt lt -180,tmp_count)
                       endwhile
                       tmp_ind1 = temporary(tmp_ind) ;destroy tmp_ind
                       if n_pos gt 0 then begin
                          tmp_ind1 = where((da_tt ge pos_min) and (da_tt le pos_max),n_da_tt1)
                          if n_da_tt1 gt 0 then tmp_ind = temporary(tmp_ind1)
                       endif
                       if n_neg gt 0 then begin
                          tmp_ind2 = where((da_tt ge neg_min) and (da_tt le neg_max),n_da_tt2)
                          if n_da_tt2 gt 0 then begin
                             if n_elements(tmp_ind) gt 0 then $
                                tmp_ind = [tmp_ind,temporary(tmp_ind2)] $
                             else $
                                tmp_ind = temporary(tmp_ind2)
                           endif
                       endif
                       if n_elements(tmp_ind) gt 0 then begin
                          da_tt  = da_tt[tmp_ind]
                          da_psi = da_psi[tmp_ind] 
                          if self.psi_spe then begin ;SPE type PSI 
                             ind_psi = where(da_tt gt 0,count_psi)
                             if count_psi gt 0 then da_psi[ind_psi] = 180.0
                             da_tt = abs(da_tt)
                          endif
                       endif else tmp_ind = temporary(da_tt)
                    endelse
                    if n_elements(da_tt) gt 0 then begin
                       daQ0 = dm_proj_spec_samp(eief,en,da_tt,da_psi,psi=psi[i],type=type,is_spe=self.psi_spe,diffetyp=(self.samp_typ eq 3),instrgeom=self.instrgeom)
                       daQ0 = reform(daQ0,3,n_elements(daQ0)/3,/overwrite)  
                       if n_elements(daQ) eq 0 then daQ = daQ0 else daQ=[[daQ],[daQ0]] 
                    endif      
                endfor
                tmp_det_tt = 0 & tmp_detpsi = 0
             endelse
             da_tt = 0 & da_psi = 0 & tmp_psi = 0
             if n_elements(daQ) ne 0 then daQV = dm_proj_view(daQ,U1,U2,U3,/no_copy,n_fold=n_fold,sr_type=self.symrot_type)
          endif else begin  ;mask dark angle
             for i=0LL,n_elements(psi)-1 do begin
                 da_tt = -(psi[i]+self.datr_psi)
                 while da_tt gt 180 do da_tt = da_tt-360
                 while da_tt lt -180 do da_tt = da_tt+360
                 if abs((abs(da_tt)-180)) le self.datr_width/2.0 then begin     ;blocking the incident beam
                    if self.instrname eq 'macs' then begin
                       qty[*,*,i] = !values.f_nan
                       err[*,*,i] = !values.f_nan
                    endif else begin
                       if self.samp_typ eq 2 then begin
                          qty[*,i] = !values.f_nan
                          err[*,i] = !values.f_nan
                       endif else begin
                          qty[*,*,i] = !values.f_nan
                          err[*,*,i] = !values.f_nan
                       endelse 
                    endelse
                 endif else begin
                    da_tt  = da_tt+self.datr_width*[-0.5,0.5]                   ;dark angle range [-0.5,0.5]*self.datr_width
                    if self.instrname eq 'macs' then begin
                       tmp_det = det_tt-76+kidney[i]  
                       ind = where(tmp_det ge da_tt[0] and tmp_det le da_tt[1],count)
                       if count ne 0 then begin
                          qty[*,ind,i] = !values.f_nan
                          err[*,ind,i] = !values.f_nan
                       endif
                    endif else begin
                       ind = where(det_tt ge da_tt[0] and det_tt le da_tt[1],count)
                       if count ne 0 then begin
                          if self.samp_typ eq 2 then begin
                             qty[ind,i] = !values.f_nan
                             err[ind,i] = !values.f_nan
                          endif else begin
                             qty[*,ind,i] = !values.f_nan
                             err[*,ind,i] = !values.f_nan
                          endelse 
                       endif 
                    endelse
                 endelse
             endfor
          endelse 
       endif

       ;Al, Cu, or Stainless Steel powder line calculation
       if ptr_valid(self.powd_qvalue) and (min_en le elastic_tol) and (max_en ge -elastic_tol) then begin  ;must be elastic
          tmp_ei = eief
          if self.instrname eq 'macs' then begin
             if self.macsintn_typ eq 0 then begin       ;'SPEC' 
                maxQ = max(dm_proj_spec_samp(Ei,Ef,det_tt,det_psi,type=0,kidney=kidney),min=minQ) 
             endif else begin                           ;'DIFF' or 'SPEC+DIFF'
                maxQ = max(dm_proj_spec_samp(Ei,Ei,det_tt,det_psi,type=0,kidney=kidney),min=minQ)
                tmp_ei = Ei[uniq(Ei,sort(Ei))]
             endelse
          endif else begin
             maxQ = max(abs(sqrt(eief/2.0721246)*sin(det_tt/2*!dtor)*2.),min=minQ)
          endelse
          if self.powd_showall then begin
             tmppsi = findgen(361)*!dtor
          endif else begin
             psimax = max(psi,min=psimin) & npsi = 11>(round(psimax-psimin)+1)
             tmppsi = (psimin-0.5+findgen(npsi)/(npsi-1)*(psimax-psimin+1))*!dtor
          endelse
          tmpcospsi = cos(tmppsi)
          tmpsinpsi = sin(temporary(tmppsi))
          ;now figure out which detectors will intercept powder cones,use 0.5 degree as threshhold
          if self.instrname eq 'macs' then begin
             tmp_kid = kidney[uniq(kidney,sort(kidney))]
             n_kid = n_elements(tmp_kid) & n_det = n_elements(det_tt)
             tmp_det_tt = (fltarr(n_kid)+1)#det_tt
             tmp_detpsi = (fltarr(n_kid)+1)#det_psi
             for i=0L,n_det-1 do tmp_det_tt[*,i] = tmp_kid-76.0+det_tt[i]
             tmp_kid = 0
          endif else begin
             tmp_det_tt = det_tt
             tmp_detpsi = det_psi
          endelse
          for i=0L,n_elements(*self.powd_qvalue)-1 do begin
              if ((*self.powd_qvalue)[i] lt minQ) then continue
              if ((*self.powd_qvalue)[i] gt maxQ) then break
              for j=0L,n_elements(tmp_ei)-1 do begin
                  k = sqrt(tmp_ei[j]/2.07212)
                  tth = asin((*self.powd_qvalue)[i]/2./k)*2/!dtor
                  if self.powd_showall then begin
                     tmptth = tth*(2*(tmp_det_tt gt 0)-1)
                     tmpdetpsi = tmp_detpsi   
                  endif else begin
                     ind = where(abs(abs(tmp_det_tt)-tth) le 0.5,count)
                     if count eq 0 then continue
                     tmptth = tth*(2*(tmp_det_tt[ind] gt 0)-1)
                     tmpdetpsi = tmp_detpsi[ind]
                  endelse
                  tmp = dm_to_string(tmptth,resolution=3)+'_'+dm_to_string(tmpdetpsi,resolution=3)
                  ind = sort(tmp)
                  ind1 = uniq(tmp[ind])
                  tmptth = tmptth[ind[ind1]]
                  tmpdetpsi = tmpdetpsi[ind[ind1]]
                  for l=0L,n_elements(tmptth)-1 do begin
                      ;in psi=0 uvw frame
                      x = k*(1.0-cos(tmptth[l]*!dtor))
                      sin2th = sin(tmptth[l]*!dtor)
                      sinpsi = sin(tmpdetpsi[l]*!dtor)
                      cospsi = cos(tmpdetpsi[l]*!dtor)
                      if self.psi_spe then begin ;SPE type detector info
                         y = -k*sin2th*cospsi
                         z = -k*sin2th*sinpsi
                      endif else begin           ;DCS type detector info
                         if sin2th eq 0 then begin
                            y = 0.0
                            z = 0.0
                         endif else begin
                            y = k*sin2th*sqrt(1-(sinpsi/sin2th)^2)
                            z = -k*sinpsi
                         endelse
                      endelse
                      ;rotate to sample frame
                      xs = x*tmpcospsi+y*tmpsinpsi
                      ys = y*tmpcospsi-x*tmpsinpsi
                      zs = replicate(z,n_elements(xs))
                      if self.powd_showall then n_fold = 1
                      Vpow = dm_proj_view(transpose([[xs],[ys],[zs]]),U1,U2,U3,n_fold=n_fold,sr_type=self.symrot_type)
                      xs = reform(Vpow[0,*]) & ys = reform(Vpow[1,*]) & zs = reform(Vpow[2,*])
                      if n_elements(lines) ne 0 then lines=[[[lines]],[[[xs],[ys],[zs]]]] $
                      else lines=[[xs],[ys],[zs]]
                  endfor      
              endfor
          endfor
          tmp_det_tt = 0 & tmp_detpsi = 0 & tmpcospsi = 0 & tmpsinpsi = 0
          if n_elements(lines) ne 0 then self.powd_line = ptr_new(lines,/no_copy)
       endif
       
       ;check folding
       fold = [-1] & fcnt = [0.0] & fdir = [0]
       if self.view_u1fold and (self.fold_type eq 0) then begin
          nfc = n_elements(*(self.view_u1foldcenter))
          nfd = n_elements(*(self.view_u1folddir))
          index = where(finite(*(self.view_u1foldcenter)),count,complement=index1,ncomplement=count1)
          if self.mrrf_type eq 1 then begin
             if count gt 0 then begin
                *(self.view_u1foldcenter) = (*(self.view_u1foldcenter))[index[0]]
                if count gt 1 then begin
                   tmp = 'Only the first u1 folding center is used when folding to both sides.'
                   if n_elements(mesg) eq 0 then mesg = [tmp,''] else mesg = [mesg,tmp,'']
                endif
             endif
             if (nfc gt 1) or (count1 gt 0) then reset_projection = 1b
          endif else begin
             if nfc ne nfd then begin
                *(self.view_u1folddir) = (dm_dialog_input('center value '+dm_to_string(*(self.view_u1foldcenter))+':',title='Set u1 folding direction',$
                       default =*(self.view_u1folddir)+1,is_droplist=1+intarr(nfc),droplist_content=ptr_new(['-','auto','+']),xsize=100,dialog_parent=self.tlb,/return_number)-1)
               if ~keyword_set(debug) then begin
                  value = (['-','auto','+'])[(*self.view_u1folddir)[0]+1]
                  for i=1,nfc-1 do value = value+','+(['-','auto','+'])[(*self.view_u1folddir)[i]+1]
                  dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU1Folddir'),select=0,set_value=[value,'auto','+']
               endif
             endif
             if count gt 0 then begin
                fold = [fold,intarr(count)]
                fcnt = [fcnt,(*(self.view_u1foldcenter))[index]]
                fdir = [fdir,(*(self.view_u1folddir))[index]]
             endif
             if count1 gt 0 then begin
                if count+count1 eq 1 then tmp = 'u1 folding center value is invalid.' $
                else tmp = 'u1 folding center value #'+dm_to_string(index1+1,separator=', ')+([' is',' are'])[count1 gt 1]+' invalid.'
                if n_elements(mesg) eq 0 then mesg = [tmp,''] else mesg = [mesg,tmp,'']
             endif
          endelse
       endif
       if self.view_u2fold and (self.fold_type eq 0) then begin
          nfc = n_elements(*(self.view_u2foldcenter))
          nfd = n_elements(*(self.view_u2folddir))
          index = where(finite(*(self.view_u2foldcenter)),count,complement=index1,ncomplement=count1)
          if self.mrrf_type eq 1 then begin
             if count gt 0 then begin
                *(self.view_u2foldcenter) = (*(self.view_u2foldcenter))[index[0]]
                if count gt 1 then begin
                   tmp = 'Only the first u2 folding center is used when folding to both sides.'
                   if n_elements(mesg) eq 0 then mesg = [tmp,''] else mesg = [mesg,tmp,'']
                endif
             endif
             if (nfc gt 1) or (count1 gt 0) then reset_projection = 1b
          endif else begin
             if nfc ne nfd then begin
                *(self.view_u2folddir) = (dm_dialog_input('center value '+dm_to_string(*(self.view_u2foldcenter))+':',title='Set u2 folding direction',$
                       default =*(self.view_u2folddir)+1,is_droplist=1+intarr(nfc),droplist_content=ptr_new(['-','auto','+']),xsize=100,dialog_parent=self.tlb,/return_number)-1)
               if ~keyword_set(debug) then begin
                  value = (['-','auto','+'])[(*self.view_u2folddir)[0]+1]
                  for i=1,nfc-1 do value = value+','+(['-','auto','+'])[(*self.view_u2folddir)[i]+1]
                  dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU2Folddir'),select=0,set_value=[value,'auto','+']
               endif
             endif
             if count gt 0 then begin
                fold = [fold,intarr(count)+1]
                fcnt = [fcnt,(*(self.view_u2foldcenter))[index]]
                fdir = [fdir,(*(self.view_u2folddir))[index]]
             endif
             if count1 gt 0 then begin
                if count+count1 eq 1 then tmp = 'u2 folding center value is invalid.' $
                else tmp = 'u2 folding center value #'+dm_to_string(index1+1,separator=', ')+([' is',' are'])[count1 gt 1]+' invalid.'
                if n_elements(mesg) eq 0 then mesg = [tmp,''] else mesg = [mesg,tmp,'']
             endif
          endelse
       endif
       if n_elements(view_u3) eq 0 then begin
          QV = QV[0:1,*,*,*]
          if n_elements(daQV) ne 0 then daQV = daQV[0:1,*]
       endif 
       if extravaxis then begin
          if self.view_u3fold and (self.fold_type eq 0) then begin
             nfc = n_elements(*(self.view_u3foldcenter))
             nfd = n_elements(*(self.view_u3folddir))
             index = where(finite(*(self.view_u3foldcenter)),count,complement=index1,ncomplement=count1)
             if self.mrrf_type eq 1 then begin
                if count gt 0 then begin
                   *(self.view_u3foldcenter) = (*(self.view_u3foldcenter))[index[0]]
                   if count gt 1 then begin
                      tmp = 'Only the first u3 folding center is used when folding to both sides.'
                      if n_elements(mesg) eq 0 then mesg = [tmp,''] else mesg = [mesg,tmp,'']
                   endif
                endif
                if (nfc gt 1) or (count1 gt 0) then reset_projection = 1b
             endif else begin
                if nfc ne nfd then begin
                   *(self.view_u3folddir) = (dm_dialog_input('center value '+dm_to_string(*(self.view_u3foldcenter))+':',title='Set u3 folding direction',$
                          default =*(self.view_u3folddir)+1,is_droplist=1+intarr(nfc),droplist_content=ptr_new(['-','auto','+']),xsize=100,dialog_parent=self.tlb,/return_number)-1)
                   if ~keyword_set(debug) then begin
                      value = (['-','auto','+'])[(*self.view_u3folddir)[0]+1]
                      for i=1,nfc-1 do value = value+','+(['-','auto','+'])[(*self.view_u3folddir)[i]+1]
                      dm_set_droplist,widget_info(self.projBas,find_by_uname='viewU3Folddir'),select=0,set_value=[value,'auto','+']
                   endif
                endif
                if count gt 0 then begin
                   fold = [fold,intarr(count)+2]
                   fcnt = [fcnt,(*(self.view_u3foldcenter))[index]]
                   fdir = [fdir,(*(self.view_u3folddir))[index]]
                endif
                if count1 gt 0 then begin
                   if count+count1 eq 1 then tmp = 'u3 folding center value is invalid.' $
                   else tmp = 'u3 folding center value #'+dm_to_string(index1+1,separator=', ')+([' is',' are'])[count1 gt 1]+' invalid.'
                   if n_elements(mesg) eq 0 then mesg = [tmp,''] else mesg = [mesg,tmp,'']
                endif
             endelse
          endif
       endif
       ;folding of viewing axis
       for i=1,n_elements(fold)-1 do begin
           xdat = QV[fold[i],*,*,*]    
           if n_elements(daQV) ne 0 then daxdat = daQV[fold[i],*]
           if ptr_valid(self.powd_line) and (~self.powd_showall) then pdxdat = (*self.powd_line)[*,fold[i],*]
           count = 0 & dacount = 0 & pdcount = 0
           case fdir[i] of 
                -1: begin    ;'-' side, unless no data to the '-' side
                    index = where(xdat gt fcnt[i],count,ncomplement=count1)
                    if n_elements(daQV) ne 0 then daindex = where(daxdat gt fcnt[i],dacount,ncomplement=dacount1)
                    if n_elements(pdxdat) ne 0 then pdindex = where(pdxdat gt fcnt[i],pdcount,ncomplement=pdcount1)
                    if count1 eq 0 then begin  ;no data to the '-' side
                       count = 0 & index = -1 & dacount = 0 & daindex = -1 & pdcount = 0 & pdindex = -1
                    endif
                    end
                 0: begin    ;auto
                    ;determine on which side of the folding center that the data has wider range
                    ;fold to greater range side   
                    index = where(xdat gt fcnt[i],count,complement=index1,ncomplement=count1)
                    if n_elements(daQV) ne 0 then daindex = where(daxdat gt fcnt[i],dacount,complement=daindex1,ncomplement=dacount1)
                    if n_elements(pdxdat) ne 0 then pdindex = where(pdxdat gt fcnt[i],pdcount,complement=pdindex1,ncomplement=pdcount1)
                    if (count ne 0) and (count1 ne 0) then begin
                       x_max = max(xdat[index],min=x_min)
                       p_ran = x_max-x_min
                       x_max = max(xdat[index1],min=x_min)
                       n_ran = x_max-x_min  
                       if p_ran ge n_ran then begin
                          index = temporary(index1) 
                          count = count1
                          if n_elements(daQV) ne 0 then begin
                             daindex = temporary(daindex1) 
                             dacount = dacount1
                          endif
                          if n_elements(pdxdat) ne 0 then begin
                             pdindex = temporary(pdindex1) 
                             pdcount = pdcount1
                          endif
                       endif else begin
                          index1 = -1 & daindex1 = -1 & pdindex1 = -1
                       endelse
                    endif else begin
                       tmp = temporary(p_ran) & tmp = temporary(n_ran) & count = 0
                    endelse
                    end
                 1: begin    ;'+' side,unless no data to the '+' side
                    index = where(xdat lt fcnt[i],count,ncomplement=count1)
                    if n_elements(daQV) ne 0 then daindex = where(daxdat lt fcnt[i],dacount,ncomplement=dacount1)
                    if n_elements(pdxdat) ne 0 then pdindex = where(pdxdat lt fcnt[i],pdcount,ncomplement=pdcount1)
                    if count1 eq 0 then begin  ;no data to the '+' side
                       count = 0 & index = -1 & dacount = 0 & daindex = -1 & pdcount = 0 & pdindex = -1
                    endif
                    end
              else:
           endcase
           if count ne 0 then begin
              xdat[index] = (2.*fcnt[i]-xdat[index]) & index = -1
              QV[fold[i],*,*,*] = temporary(xdat)
           endif
           if dacount ne 0 then begin
              daxdat[daindex] = (2.*fcnt[i]-daxdat[daindex]) & daindex = -1
              daQV[fold[i],*] = temporary(daxdat)
           endif
           if pdcount ne 0 then begin
              pdxdat[pdindex] = (2.*fcnt[i]-pdxdat[pdindex]) & pdindex = -1
              (*self.powd_line)[*,fold[i],*] = temporary(pdxdat)
           endif
       endfor
       end
    endcase

    if (self.intn_typ ge 4) and (self.intn_typ ne (n_elements(self.intn_typ_str)-1)) then begin
       if self.samp_typ gt 0 then Q = sqrt(total(Q^2,1))
       case self.intn_typ of 
            4: begin    ;GDOS
               if dwu2 eq 0 then factor = 0.48259649/Q^2 $
               else factor = 0.48259649/Q^2*exp(dwu2*Q^2)
               end
            5: begin    ;VDOS
               tthmax = max(det_tt,min=tthmin)
               if self.instrgeom eq 0 then efei = eief-en $
               else efei = eief+en
               root = sqrt(eief*efei)
               q4maxmin = (eief+efei-2.0*root*cos(tthmax*!dtor))^2-(eief+efei-2.0*root*cos(tthmin*!dtor))^2
               if dwu2 eq 0 then factor = (1./q4maxmin)#sin(det_tt*!dtor) $
               else factor = ((1./q4maxmin)#sin(det_tt*!dtor))*exp(dwu2*Q^2)
               end
            6: begin    ;J(Q,y)
               factor = 2.07212*1.008665*2/Mjqy*Q
               if self.samp_typ eq 0 then y = ((en#(fltarr(ndet)+1))-2.07212*1.008665/Mjqy*Q^2)/factor
               end
            else: factor = 1.0
       endcase
       qty = qty*factor
       err = err*temporary(factor)
    endif

    ;user function executed after projection
    if (strlen(userf_str) ne 0) and (self.userf_before eq 0) then begin
       if self.userfwarning then begin
          if lmgr(/runtime) then usertmp = 'User Macro' else usertmp = 'User Macro/Function/Procedure'
          if n_elements(mesg) eq 0 then mesg = [usertmp+' is specified.',''] $
          else mesg = [usertmp+' is specified.','',mesg]
          self.userfwarning = 0b
       endif
       ok = execute(userf_str,1,1)
       if ~ok then begin
          errmsg = strmid(!error_state.msg,0,strlen(!error_state.msg)-1)+' in the user macro.'
          ok = dialog_message(errmsg,/error,dialog_parent=self.tlb,/center)
          return  ;error encountered
       endif
       if n_elements(err) ne n_elements(qty) then begin ;in case qty and err are not both defined in user functions
          if n_elements(err) eq 1 then begin
             err = replicate(err,size(qty,/dimension))
          endif else begin
             help,qty,err,output=tmp
             ok = dialog_message(['Incompatible array sizes of intensity and error bar, please check the user macro.','',tmp],/error,dialog_parent=self.tlb,/center)
             return
          endelse
       endif
    endif

    if self.instrname eq 'macs' then begin
       if ptr_valid(self.dataStrPtr) then begin
          if size(qty,/n_dimensions) eq 2 then begin
             qty = reform(qty,[size(qty,/dimensions),1],/overwrite)
             err = reform(err,[size(err,/dimensions),1],/overwrite)
          endif
          qty = transpose(qty,[0,2,1])  ;[2,20,ndat]->[2,ndat,ndet]
          err = transpose(err,[0,2,1])
       endif
    endif

    ;store projection
    if self.samp_typ eq 0 then begin        ;powder
       if n_elements(kidney) ne 0 then begin
          if self.intn_typ eq (n_elements(self.intn_typ_str)-2) then $  ;J(Q,y), not used yet
             self.projStrPtr = ptr_new({Q:temporary(Q),en:temporary(en),two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err),ewid:temporary(ewid),y:temporary(y),kidney:temporary(kidney),psi:temporary(psi),temperature:temporary(data_temperature),hfield:temporary(data_hfield),specify:temporary(data_specify)}) $
          else $
             self.projStrPtr = ptr_new({Q:temporary(Q),en:temporary(en),two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err),ewid:temporary(ewid),kidney:temporary(kidney),psi:temporary(psi),temperature:temporary(data_temperature),hfield:temporary(data_hfield),specify:temporary(data_specify)})
       endif else begin
          if self.intn_typ eq (n_elements(self.intn_typ_str)-2) then $  ;J(Q,y)
             self.projStrPtr = ptr_new({Q:temporary(Q),en:temporary(en),two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err),ewid:temporary(ewid),time:temporary(time),y:temporary(y)}) $
          else $
             self.projStrPtr = ptr_new({Q:temporary(Q),en:temporary(en),two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err),ewid:temporary(ewid),time:temporary(time)})
       endelse
    endif else begin                        ;single crystal    
       if n_elements(daQV) ne 0 then begin
          if n_elements(kidney) ne 0 then begin
             self.projStrPtr = ptr_new({V:temporary(QV),en:temporary(en),ewid:temporary(ewid),two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err),kidney:temporary(kidney),psi:temporary(psi),temperature:temporary(data_temperature),hfield:temporary(data_hfield),specify:temporary(data_specify),daQV:temporary(daQV),daen:temporary(da_en)})
          endif else begin
             self.projStrPtr = ptr_new({V:temporary(QV),en:temporary(en),ewid:temporary(ewid),two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err),temperature:temporary(data_temperature),daQV:temporary(daQV)})
          endelse
       endif else begin
          if n_elements(kidney) ne 0 then begin
             self.projStrPtr = ptr_new({V:temporary(QV),en:temporary(en),ewid:temporary(ewid),two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err),kidney:temporary(kidney),psi:temporary(psi),temperature:temporary(data_temperature),hfield:temporary(data_hfield),specify:temporary(data_specify)})
          endif else begin
             self.projStrPtr = ptr_new({V:temporary(QV),en:temporary(en),ewid:temporary(ewid),two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err),temperature:temporary(data_temperature)})
          endelse
       endelse
    endelse

    ;allow plots
    if (~ keyword_set(debug)) then $
       self->set_display_button  $
    else $
       print,'**Projection calculation finished in ',systime(/sec)-current,' sec.'
    if ptr_valid(self.dataStrPtr) and (self.bin_avgsum eq 1) and (self.e_bin[self.samp_typ] gt 1) and (self.samp_typ lt 2)then begin
       if n_elements(mesg) eq 0 then mesg='The current binning method is SUM.' $
       else mesg = [mesg,'The current binning method is SUM.']
    endif   
    if n_elements(mesg) ne 0 then ok = dialog_message([mesg,'','This message is only a reminder.'],dialog_parent=self.tlb,/center)
    if keyword_set(reset_projection) then self->reset_projection,/reset,/foldonly
    self.error = 0       ;clear the error flag
end